from __future__ import unicode_literals

from django.contrib import admin

from .models import Message


@admin.register(Message)
class MessageAdmin(admin.ModelAdmin):
    list_display = ('label', 'enabled', 'start_datetime', 'end_datetime')



from .forms import (
    POP3EmailSetupForm, IMAPEmailSetupForm, SaneScannerSetupForm,
    SaneScannerUploadForm, StagingFolderSetupForm, StagingUploadForm,
    WatchFolderSetupForm, WebFormSetupForm, WebFormUploadForm
)
from .literals import (
    SOURCE_CHOICE_EMAIL_IMAP, SOURCE_CHOICE_EMAIL_POP3,
    SOURCE_CHOICE_SANE_SCANNER, SOURCE_CHOICE_STAGING, SOURCE_CHOICE_WATCH,
    SOURCE_CHOICE_WEB_FORM
)
from .models import (
    IMAPEmail, POP3Email, SaneScanner, StagingFolderSource, WatchFolderSource,
    WebFormSource
)


def get_class(source_type):
    if source_type == SOURCE_CHOICE_WEB_FORM:
        return WebFormSource
    elif source_type == SOURCE_CHOICE_STAGING:
        return StagingFolderSource
    elif source_type == SOURCE_CHOICE_WATCH:
        return WatchFolderSource
    elif source_type == SOURCE_CHOICE_EMAIL_POP3:
        return POP3Email
    elif source_type == SOURCE_CHOICE_EMAIL_IMAP:
        return IMAPEmail
    elif source_type == SOURCE_CHOICE_SANE_SCANNER:
        return SaneScanner


def get_form_class(source_type):
    if source_type == SOURCE_CHOICE_WEB_FORM:
        return WebFormSetupForm
    elif source_type == SOURCE_CHOICE_STAGING:
        return StagingFolderSetupForm
    elif source_type == SOURCE_CHOICE_WATCH:
        return WatchFolderSetupForm
    elif source_type == SOURCE_CHOICE_EMAIL_POP3:
        return POP3EmailSetupForm
    elif source_type == SOURCE_CHOICE_EMAIL_IMAP:
        return IMAPEmailSetupForm
    elif source_type == SOURCE_CHOICE_SANE_SCANNER:
        return SaneScannerSetupForm


def get_upload_form_class(source_type):
    if source_type == SOURCE_CHOICE_WEB_FORM:
        return WebFormUploadForm
    elif source_type == SOURCE_CHOICE_STAGING:
        return StagingUploadForm
    elif source_type == SOURCE_CHOICE_SANE_SCANNER:
        return SaneScannerUploadForm



from __future__ import unicode_literals

from django.template import Library

from ..classes import Setting

register = Library()


@register.simple_tag
def smart_setting(global_name):
    return Setting.get(global_name=global_name).value


@register.simple_tag
def smart_settings_check_changed():
    return Setting.check_changed()



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group

from .literals import (
    TEST_CASE_SUPERUSER_EMAIL, TEST_CASE_SUPERUSER_PASSWORD, TEST_CASE_SUPERUSER_USERNAME,
    TEST_CASE_GROUP_NAME, TEST_GROUP_NAME_EDITED, TEST_CASE_USER_EMAIL,
    TEST_CASE_USER_PASSWORD, TEST_CASE_USER_USERNAME, TEST_GROUP_NAME,
    TEST_USER_EMAIL, TEST_USER_USERNAME, TEST_USER_USERNAME_EDITED,
    TEST_USER_PASSWORD, TEST_USER_PASSWORD_EDITED
)


class GroupAPIViewTestMixin(object):
    def _request_test_group_create_api_view(self):
        result = self.post(
            viewname='rest_api:group-list', data={
                'name': TEST_GROUP_NAME
            }
        )
        if 'id' in result.json():
            self.test_group = Group.objects.get(pk=result.json()['id'])

        return result

    def _request_test_group_delete_api_view(self):
        return self.delete(
            viewname='rest_api:group-detail', kwargs={'pk': self.test_group.pk}
        )

    def _request_test_group_edit_patch_api_view(self):
        return self.patch(
            viewname='rest_api:group-detail', kwargs={'pk': self.test_group.pk},
            data={
                'name': TEST_GROUP_NAME_EDITED
            }
        )

    def _request_test_group_edit_put_api_view(self):
        return self.put(
            viewname='rest_api:group-detail', kwargs={'pk': self.test_group.pk},
            data={
                'name': TEST_GROUP_NAME_EDITED
            }
        )


class GroupTestMixin(object):
    def _create_test_group(self):
        self.test_group = Group.objects.create(name=TEST_GROUP_NAME)

    def _edit_test_group(self):
        self.test_group.name = TEST_GROUP_NAME_EDITED
        self.test_group.save()


class GroupViewTestMixin(object):
    def _request_test_group_create_view(self):
        reponse = self.post(
            viewname='user_management:group_create', data={
                'name': TEST_GROUP_NAME
            }
        )
        self.test_group = Group.objects.filter(name=TEST_GROUP_NAME).first()
        return reponse

    def _request_test_group_delete_view(self):
        return self.post(
            viewname='user_management:group_delete', kwargs={
                'pk': self.test_group.pk
            }
        )

    def _request_test_group_edit_view(self):
        return self.post(
            viewname='user_management:group_edit', kwargs={
                'pk': self.test_group.pk
            }, data={
                'name': TEST_GROUP_NAME_EDITED
            }
        )

    def _request_test_group_list_view(self):
        return self.get(viewname='user_management:group_list')

    def _request_test_group_members_view(self):
        return self.get(
            viewname='user_management:group_members',
            kwargs={'pk': self.test_group.pk}
        )


class UserAPIViewTestMixin(object):
    def _request_test_user_create_api_view(self):
        result = self.post(
            viewname='rest_api:user-list', data={
                'email': TEST_USER_EMAIL, 'password': TEST_USER_PASSWORD,
                'username': TEST_USER_USERNAME,
            }
        )
        if 'id' in result.json():
            self.test_user = get_user_model().objects.get(pk=result.json()['id'])

        return result

    def _request_test_user_create_api_view_extra_data(self):
        result = self.post(
            viewname='rest_api:user-list', data={
                'email': TEST_USER_EMAIL, 'password': TEST_USER_PASSWORD,
                'username': TEST_USER_USERNAME,
                'groups_pk_list': self.test_group.pk
            }
        )

        if 'id' in result.json():
            self.test_user = get_user_model().objects.get(pk=result.json()['id'])

        return result

    def _request_test_user_delete_api_view(self):
        return self.delete(
            viewname='rest_api:user-detail',
            kwargs={'pk': self.test_user.pk}
        )

    def _request_test_user_edit_patch_api_view(self):
        return self.patch(
            viewname='rest_api:user-detail',
            kwargs={'pk': self.test_user.pk},
            data={'username': TEST_USER_USERNAME_EDITED}
        )

    def _request_test_user_edit_put_api_view(self):
        return self.put(
            viewname='rest_api:user-detail',
            kwargs={'pk': self.test_user.pk},
            data={'username': TEST_USER_USERNAME_EDITED}
        )

    def _request_test_user_group_add_api_view(self):
        return self.post(
            viewname='rest_api:users-group-list', kwargs={
                'pk': self.test_user.pk
            }, data={
                'group_pk_list': '{}'.format(self.test_group.pk)
            }
        )

    def _request_test_user_group_list_api_view(self):
        return self.get(
            viewname='rest_api:users-group-list', kwargs={
                'pk': self.test_user.pk
            }
        )

    def _request_test_user_password_change_api_view(self):
        result = self.patch(
            viewname='rest_api:user-detail', kwargs={
                'pk': self.test_user.pk
            }, data={
                'password': TEST_USER_PASSWORD_EDITED,
            }
        )

        self.test_user.cleartext_password = TEST_USER_PASSWORD_EDITED

        return result


class UserTestCaseMixin(object):
    """
    This TestCaseMixin is used to create a user and group to execute the
    test case, these are used to just create an identity which is required by
    most of the code in the project, these are not meant to be acted upon
    (edited, deleted, etc). To create a test users or groups to modify, use
    the UserTestMixin instead and the respective test_user and test_group.
    The user and group created by this mixin will be prepended with
    _test_case_{...}. The _test_case_user and _test_case_group are meant
    to be used by other test case mixins like the ACLs test case mixin which
    adds shorthand methods to create ACL entries to test access control.
    """
    auto_login_superuser = False
    auto_login_user = True
    create_test_case_superuser = False
    create_test_case_user = True

    def setUp(self):
        super(UserTestCaseMixin, self).setUp()
        if self.create_test_case_user:
            self._create_test_case_user()
            self._create_test_case_group()
            self._test_case_group.user_set.add(self._test_case_user)

            if self.auto_login_user:
                self.login_user()

        if self.create_test_case_superuser:
            self._create_test_case_superuser()

            if self.auto_login_superuser:
                self.login_superuser()

    def tearDown(self):
        self.client.logout()
        super(UserTestCaseMixin, self).tearDown()

    def _create_test_case_group(self):
        self._test_case_group = Group.objects.create(name=TEST_CASE_GROUP_NAME)

    def _create_test_case_superuser(self):
        self._test_case_superuser = get_user_model().objects.create_superuser(
            username=TEST_CASE_SUPERUSER_USERNAME, email=TEST_CASE_SUPERUSER_EMAIL,
            password=TEST_CASE_SUPERUSER_PASSWORD
        )
        self._test_case_superuser.cleartext_password = TEST_CASE_SUPERUSER_PASSWORD

    def _create_test_case_user(self):
        self._test_case_user = get_user_model().objects.create_user(
            username=TEST_CASE_USER_USERNAME, email=TEST_CASE_USER_EMAIL,
            password=TEST_CASE_USER_PASSWORD
        )
        self._test_case_user.cleartext_password = TEST_CASE_USER_PASSWORD

    def login(self, *args, **kwargs):
        return self.client.login(*args, **kwargs)

    def login_superuser(self):
        return self.login(
            username=TEST_CASE_SUPERUSER_USERNAME,
            password=TEST_CASE_SUPERUSER_PASSWORD
        )

    def login_user(self):
        return self.login(
            username=TEST_CASE_USER_USERNAME, password=TEST_CASE_USER_PASSWORD
        )

    def logout(self):
        self.client.logout()


class UserTestMixin(object):
    def _create_test_superuser(self):
        self.test_superuser = get_user_model().objects.create_superuser(
            username=TEST_CASE_SUPERUSER_USERNAME, email=TEST_CASE_SUPERUSER_EMAIL,
            password=TEST_CASE_SUPERUSER_PASSWORD
        )
        self.test_superuser.cleartext_password = TEST_USER_PASSWORD

    def _create_test_user(self):
        total_test_users = len(self.test_users)
        username = '{}_{}'.format(TEST_USER_USERNAME, total_test_users)

        self.test_user = get_user_model().objects.create_user(
            username=username, email=TEST_USER_EMAIL,
            password=TEST_USER_PASSWORD
        )
        self.test_user.cleartext_password = TEST_USER_PASSWORD
        self.test_users.append(self.test_user)

    def setUp(self):
        super(UserTestMixin, self).setUp()
        self.test_users = []


class UserViewTestMixin(object):
    def _request_current_user_details_view(self):
        return self.get(viewname='user_management:current_user_details')

    def _request_test_superuser_delete_view(self):
        return self.post(
            viewname='user_management:user_delete',
            kwargs={'pk': self.test_superuser.pk}
        )

    def _request_test_superuser_detail_view(self):
        return self.get(
            viewname='user_management:user_details',
            kwargs={'pk': self.test_superuser.pk}
        )

    def _request_test_user_create_view(self):
        reponse = self.post(
            viewname='user_management:user_create', data={
                'username': TEST_USER_USERNAME,
                'password': TEST_USER_PASSWORD
            }
        )

        self.test_user = get_user_model().objects.filter(
            username=TEST_USER_USERNAME
        ).first()
        return reponse

    def _request_test_user_delete_view(self):
        return self.post(
            viewname='user_management:user_delete',
            kwargs={'pk': self.test_user.pk}
        )

    def _request_test_user_delete_multiple_view(self):
        return self.post(
            viewname='user_management:user_multiple_delete', data={
                'id_list': self.test_user.pk
            }
        )

    def _request_test_user_detail_view(self):
        return self.get(
            viewname='user_management:user_details', kwargs={
                'pk': self.test_user.pk
            }
        )

    def _request_test_user_edit_view(self):
        return self.post(
            viewname='user_management:user_edit', kwargs={
                'pk': self.test_user.pk
            }, data={
                'username': TEST_USER_USERNAME_EDITED
            }
        )

    def _request_test_user_groups_view(self):
        return self.get(
            viewname='user_management:user_groups',
            kwargs={'pk': self.test_user.pk}
        )



from __future__ import unicode_literals

from rest_framework import status

from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..models import Index
from ..permissions import (
    permission_document_indexing_create, permission_document_indexing_delete,
    permission_document_indexing_view
)

from .literals import TEST_INDEX_LABEL, TEST_INDEX_SLUG
from .mixins import IndexTestMixin


class DocumentIndexingAPIViewTestMixin(object):
    def _request_test_index_create_api_view(self):
        return self.post(
            viewname='rest_api:index-list', data={
                'label': TEST_INDEX_LABEL, 'slug': TEST_INDEX_SLUG,
                'document_types': self.test_document_type.pk
            }
        )

    def _request_test_index_delete_api_view(self):
        return self.delete(
            viewname='rest_api:index-detail', kwargs={
                'pk': self.test_index.pk
            }
        )

    def _request_test_index_detail_api_view(self):
        return self.get(
            viewname='rest_api:index-detail', kwargs={
                'pk': self.test_index.pk
            }
        )


class DocumentIndexingAPITestCase(
    IndexTestMixin, DocumentIndexingAPIViewTestMixin, DocumentTestMixin,
    BaseAPITestCase
):
    auto_upload_document = False

    def test_index_create_api_view_no_permission(self):
        response = self._request_test_index_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.assertEqual(Index.objects.count(), 0)

    def test_index_create_api_view_with_permission(self):
        self.grant_permission(permission=permission_document_indexing_create)

        response = self._request_test_index_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        index = Index.objects.first()

        self.assertEqual(response.data['id'], index.pk)
        self.assertEqual(response.data['label'], index.label)

        self.assertEqual(Index.objects.count(), 1)
        self.assertEqual(index.label, TEST_INDEX_LABEL)

    def test_index_delete_api_view_no_permission(self):
        self._create_test_index()

        response = self._request_test_index_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertTrue(self.test_index in Index.objects.all())

    def test_index_delete_api_view_with_access(self):
        self._create_test_index()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_delete
        )

        response = self._request_test_index_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertTrue(self.test_index not in Index.objects.all())

    def test_index_detail_api_view_no_access(self):
        self._create_test_index()

        response = self._request_test_index_detail_api_view()

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertTrue('id' not in response.data)

    def test_index_detail_api_view_with_access(self):
        self._create_test_index()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_view
        )

        response = self._request_test_index_detail_api_view()

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['id'], self.test_index.pk
        )



from __future__ import unicode_literals

from mayan.apps.common.tests.base import GenericViewTestCase

from ..permissions import (
    permission_cache_purge, permission_cache_view
)

from .mixins import CacheTestMixin, CacheViewTestMixin


class CacheViewTestCase(
    CacheTestMixin, CacheViewTestMixin, GenericViewTestCase
):
    def test_cache_list_view_with_no_permission(self):
        self._create_test_cache()

        response = self._request_test_cache_list_view()
        self.assertNotContains(
            response=response, text=self.test_cache.label, status_code=200
        )

    def test_cache_list_view_with_access(self):
        self._create_test_cache()

        self.grant_access(
            obj=self.test_cache, permission=permission_cache_view
        )

        response = self._request_test_cache_list_view()
        self.assertContains(
            response=response, text=self.test_cache.label, status_code=200
        )

    def test_cache_purge_view_no_permissions(self):
        self._create_test_cache()
        self._create_test_cache_partition()
        self._create_test_cache_partition_file()

        cache_total_size = self.test_cache.get_total_size()

        response = self._request_test_cache_purge_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(cache_total_size, self.test_cache.get_total_size())

    def test_cache_purge_view_with_access(self):
        self._create_test_cache()
        self._create_test_cache_partition()
        self._create_test_cache_partition_file()

        self.grant_access(
            obj=self.test_cache, permission=permission_cache_purge
        )

        cache_total_size = self.test_cache.get_total_size()

        response = self._request_test_cache_purge_view()
        self.assertEqual(response.status_code, 302)

        self.assertNotEqual(cache_total_size, self.test_cache.get_total_size())

    def test_cache_multiple_purge_view_no_permissions(self):
        self._create_test_cache()
        self._create_test_cache_partition()
        self._create_test_cache_partition_file()

        cache_total_size = self.test_cache.get_total_size()

        response = self._request_test_cache_multiple_purge_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(cache_total_size, self.test_cache.get_total_size())

    def test_cache_multiple_purge_view_with_access(self):
        self._create_test_cache()
        self._create_test_cache_partition()
        self._create_test_cache_partition_file()

        self.grant_access(
            obj=self.test_cache, permission=permission_cache_purge
        )

        cache_total_size = self.test_cache.get_total_size()

        response = self._request_test_cache_multiple_purge_view()
        self.assertEqual(response.status_code, 302)

        self.assertNotEqual(cache_total_size, self.test_cache.get_total_size())



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from drf_yasg import openapi

import mayan
from mayan.apps.common.settings import setting_project_title

openapi_info = openapi.Info(
    title=_('%s API') % setting_project_title.value,
    default_version='v2',
    description=mayan.__description__,
    license=openapi.License(name=mayan.__license__),
)



from __future__ import unicode_literals

from django.db import migrations, models


def operation_rename_duplicates(apps, schema_editor):
    StatisticResult = apps.get_model(
        app_label='mayan_statistics', model_name='StatisticResult'
    )
    slugs = []

    for entry in StatisticResult.objects.using(schema_editor.connection.alias).all():
        if entry.slug in slugs:
            counter = 1
            while(True):
                attempt = '{}_{}'.format(entry.slug, counter)
                if attempt not in slugs:
                    break
                else:
                    counter = counter + 1

            entry.slug = attempt
            entry.save()

        slugs.append(entry.slug)


class Migration(migrations.Migration):

    dependencies = [
        ('mayan_statistics', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(
            code=operation_rename_duplicates,
            reverse_code=migrations.RunPython.noop
        ),
        migrations.AlterField(
            model_name='statisticresult',
            name='slug',
            field=models.SlugField(unique=True, verbose_name='Slug'),
        ),
    ]



from __future__ import unicode_literals

from mayan.apps.dependencies.classes import JavaScriptDependency

JavaScriptDependency(
    module=__name__, name='chart.js', static_folder='statistics',
    version_string='=2.7.2'
)



from __future__ import absolute_import, unicode_literals

from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.permissions import (
    permission_document_create, permission_document_new_version
)
from mayan.apps.navigation.classes import Link

from .literals import (
    SOURCE_CHOICE_WEB_FORM, SOURCE_CHOICE_EMAIL_IMAP, SOURCE_CHOICE_EMAIL_POP3,
    SOURCE_CHOICE_SANE_SCANNER, SOURCE_CHOICE_STAGING, SOURCE_CHOICE_WATCH
)
from .permissions import (
    permission_sources_setup_create, permission_sources_setup_delete,
    permission_sources_setup_edit, permission_sources_setup_view
)


def condition_check_document_creation_acls(context):
    AccessControlList = apps.get_model(
        app_label='acls', model_name='AccessControlList'
    )
    DocumentType = apps.get_model(
        app_label='documents', model_name='DocumentType'
    )

    queryset = AccessControlList.objects.restrict_queryset(
        permission=permission_document_create,
        queryset=DocumentType.objects.all(), user=context['user']
    )

    if queryset:
        return True


def document_new_version_not_blocked(context):
    NewVersionBlock = apps.get_model(
        app_label='checkouts', model_name='NewVersionBlock'
    )

    return not NewVersionBlock.objects.is_blocked(context['object'])


link_document_create_multiple = Link(
    condition=condition_check_document_creation_acls,
    icon_class_path='mayan.apps.sources.icons.icon_document_create_multiple',
    text=_('New document'),
    view='sources:document_create_multiple'
)
link_setup_sources = Link(
    icon_class_path='mayan.apps.sources.icons.icon_setup_sources',
    permissions=(permission_sources_setup_view,), text=_('Sources'),
    view='sources:setup_source_list'
)
link_setup_source_create_imap_email = Link(
    args='"%s"' % SOURCE_CHOICE_EMAIL_IMAP,
    icon_class_path='mayan.apps.sources.icons.icon_source_create',
    permissions=(permission_sources_setup_create,),
    text=_('Add new IMAP email'), view='sources:setup_source_create',
)
link_setup_source_create_pop3_email = Link(
    args='"%s"' % SOURCE_CHOICE_EMAIL_POP3,
    icon_class_path='mayan.apps.sources.icons.icon_source_create',
    permissions=(permission_sources_setup_create,),
    text=_('Add new POP3 email'), view='sources:setup_source_create',
)
link_setup_source_create_staging_folder = Link(
    args='"%s"' % SOURCE_CHOICE_STAGING,
    icon_class_path='mayan.apps.sources.icons.icon_source_create',
    permissions=(permission_sources_setup_create,),
    text=_('Add new staging folder'), view='sources:setup_source_create',
)
link_setup_source_create_watch_folder = Link(
    args='"%s"' % SOURCE_CHOICE_WATCH,
    icon_class_path='mayan.apps.sources.icons.icon_source_create',
    permissions=(permission_sources_setup_create,),
    text=_('Add new watch folder'), view='sources:setup_source_create',
)
link_setup_source_create_webform = Link(
    args='"%s"' % SOURCE_CHOICE_WEB_FORM,
    icon_class_path='mayan.apps.sources.icons.icon_source_create',
    permissions=(permission_sources_setup_create,),
    text=_('Add new webform source'), view='sources:setup_source_create',
)
link_setup_source_create_sane_scanner = Link(
    args='"%s"' % SOURCE_CHOICE_SANE_SCANNER,
    icon_class_path='mayan.apps.sources.icons.icon_source_create',
    permissions=(permission_sources_setup_create,),
    text=_('Add new SANE scanner'), view='sources:setup_source_create',
)
link_setup_source_delete = Link(
    args=('resolved_object.pk',),
    icon_class_path='mayan.apps.sources.icons.icon_setup_source_delete',
    permissions=(permission_sources_setup_delete,), tags='dangerous',
    text=_('Delete'), view='sources:setup_source_delete',
)
link_setup_source_edit = Link(
    args=('resolved_object.pk',),
    icon_class_path='mayan.apps.sources.icons.icon_setup_source_edit',
    permissions=(permission_sources_setup_edit,), text=_('Edit'),
    view='sources:setup_source_edit',
)
link_source_list = Link(
    icon_class_path='mayan.apps.sources.icons.icon_source_list',
    permissions=(permission_sources_setup_view,), text=_('Document sources'),
    view='sources:setup_web_form_list'
)
link_staging_file_delete = Link(
    args=('source.pk', 'object.encoded_filename',), keep_query=True,
    icon_class_path='mayan.apps.sources.icons.icon_staging_file_delete',
    permissions=(permission_document_new_version, permission_document_create),
    tags='dangerous', text=_('Delete'), view='sources:staging_file_delete',
)
link_document_version_upload = Link(
    args='resolved_object.pk', condition=document_new_version_not_blocked,
    icon_class_path='mayan.apps.sources.icons.icon_document_version_upload',
    permissions=(permission_document_new_version,),
    text=_('Upload new version'), view='sources:document_version_upload',
)
link_setup_source_logs = Link(
    args=('resolved_object.pk',),
    permissions=(permission_sources_setup_view,), text=_('Logs'),
    view='sources:setup_source_logs',
)
link_setup_source_check_now = Link(
    args=('resolved_object.pk',),
    icon_class_path='mayan.apps.sources.icons.icon_setup_source_check_now',
    permissions=(permission_sources_setup_view,), text=_('Check now'),
    view='sources:setup_source_check',
)



from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dynamic_search', '0002_auto_20150920_0202'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='recentsearch',
            name='user',
        ),
        migrations.DeleteModel(
            name='RecentSearch',
        ),
    ]



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sources', '0007_emailbasemodel_metadata_attachment_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailbasemodel',
            name='metadata_attachment_name',
            field=models.CharField(
                default='metadata.yaml', help_text='Name of the attachment '
                'that will contains the metadata type names and value pairs '
                'to be assigned to the rest of the downloaded attachments. '
                'Note: This attachment has to be the first attachment.',
                max_length=128, verbose_name='Metadata attachment name'
            ),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('events', '0007_auto_20170802_0823'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eventsubscription',
            name='stored_event_type',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='event_subscriptions',
                to='events.StoredEventType', verbose_name='Event type'
            ),
        ),
    ]



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from .classes import MailerBackend

__all__ = ('DjangoFileBased', 'DjangoSMTP')


class DjangoSMTP(MailerBackend):
    """
    Backend that wraps Django's SMTP backend
    """
    class_fields = (
        'host', 'port', 'use_tls', 'use_ssl', 'username', 'password'
    )
    class_path = 'django.core.mail.backends.smtp.EmailBackend'
    field_order = (
        'host', 'port', 'use_tls', 'use_ssl', 'username', 'password', 'from'
    )
    fields = {
        'from': {
            'label': _('From'),
            'class': 'django.forms.CharField', 'default': '',
            'help_text': _(
                'The sender\'s address. Some system will refuse to send '
                'messages if this value is not set.'
            ), 'kwargs': {
                'max_length': 48
            }, 'required': False
        }, 'host': {
            'label': _('Host'),
            'class': 'django.forms.CharField', 'default': 'localhost',
            'help_text': _('The host to use for sending email.'),
            'kwargs': {
                'max_length': 48
            }, 'required': False
        }, 'port': {
            'label': _('Port'),
            'class': 'django.forms.IntegerField', 'default': 25,
            'help_text': _('Port to use for the SMTP server.'),
            'required': False
        }, 'use_tls': {
            'label': _('Use TLS'),
            'class': 'django.forms.BooleanField', 'default': False,
            'help_text': _(
                'Whether to use a TLS (secure) connection when talking to '
                'the SMTP server. This is used for explicit TLS connections, '
                'generally on port 587.'
            ), 'required': False
        }, 'use_ssl': {
            'label': _('Use SSL'),
            'class': 'django.forms.BooleanField', 'default': False,
            'help_text': _(
                'Whether to use an implicit TLS (secure) connection when '
                'talking to the SMTP server. In most email documentation '
                'this type of TLS connection is referred to as SSL. It is '
                'generally used on port 465. If you are experiencing '
                'problems, see the explicit TLS setting "Use TLS". Note '
                'that "Use TLS" and "Use SSL" are mutually exclusive, '
                'so only set one of those settings to True.'
            ), 'required': False
        }, 'username': {
            'label': _('Username'),
            'class': 'django.forms.CharField', 'default': '',
            'help_text': _(
                'Username to use for the SMTP server. If empty, '
                'authentication won\'t attempted.'
            ), 'kwargs': {
                'max_length': 254
            }, 'required': False
        }, 'password': {
            'label': _('Password'),
            'class': 'django.forms.CharField', 'default': '',
            'help_text': _(
                'Password to use for the SMTP server. This setting is used '
                'in conjunction with the username when authenticating to '
                'the SMTP server. If either of these settings is empty, '
                'authentication won\'t be attempted.'
            ), 'kwargs': {
                'max_length': 192
            }, 'required': False
        },
    }
    label = _('Django SMTP backend')
    widgets = {
        'password': {
            'class': 'django.forms.widgets.PasswordInput',
            'kwargs': {
                'render_value': True
            }
        }
    }


class DjangoFileBased(MailerBackend):
    """
    Mailing backend that wraps Django's file based email backend
    """
    class_fields = ('file_path',)
    class_path = 'django.core.mail.backends.filebased.EmailBackend'
    field_order = (
        'file_path', 'from'
    )
    fields = {
        'file_path': {
            'label': _('File path'),
            'class': 'django.forms.CharField', 'kwargs': {
                'max_length': 48
            }
        }, 'from': {
            'label': _('From'),
            'class': 'django.forms.CharField', 'default': '',
            'help_text': _(
                'The sender\'s address. Some system will refuse to send '
                'messages if this value is not set.'
            ), 'kwargs': {
                'max_length': 48
            }, 'required': False
        }
    }
    label = _('Django file based backend')



from __future__ import unicode_literals

from django.test import override_settings

from mayan.apps.common.tests.base import GenericViewTestCase

from ..links import (
    link_api, link_api_documentation, link_api_documentation_redoc
)


class RESTAPILinkTestCase(GenericViewTestCase):
    @override_settings(REST_API_DISABLE_LINKS=False)
    def test_api_link_false(self):
        response = self.get(viewname='common:tools_list')

        self.assertContains(
            response=response, text=link_api.text
        )

    @override_settings(REST_API_DISABLE_LINKS=True)
    def test_api_link_true(self):
        response = self.get(viewname='common:tools_list')

        self.assertNotContains(
            response=response, text=link_api.text
        )

    @override_settings(REST_API_DISABLE_LINKS=False)
    def test_redoc_documentation_link_false(self):
        response = self.get(viewname='common:tools_list')

        self.assertContains(
            response=response, text=link_api_documentation_redoc.text
        )

    @override_settings(REST_API_DISABLE_LINKS=True)
    def test_redoc_documentation_link_true(self):
        response = self.get(viewname='common:tools_list')

        self.assertNotContains(
            response=response, text=link_api_documentation_redoc.text
        )

    @override_settings(REST_API_DISABLE_LINKS=False)
    def test_swagger_documentation_link_false(self):
        response = self.get(viewname='common:tools_list')

        self.assertContains(
            response=response, text=link_api_documentation.text
        )

    @override_settings(REST_API_DISABLE_LINKS=True)
    def test_swagger_documentation_link_true(self):
        response = self.get(viewname='common:tools_list')

        self.assertNotContains(
            response=response, text=link_api_documentation.text
        )



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(label=_('Document checkout'), name='checkouts')

permission_document_check_in = namespace.add_permission(
    label=_('Check in documents'), name='checkin_document'
)
permission_document_check_in_override = namespace.add_permission(
    label=_('Forcefully check in documents'), name='checkin_document_override'
)
permission_document_check_out = namespace.add_permission(
    label=_('Check out documents'), name='checkout_document'
)
permission_document_check_out_detail_view = namespace.add_permission(
    label=_('Check out details view'), name='checkout_detail_view'
)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link
from mayan.apps.navigation.utils import get_cascade_condition

from .permissions import (
    permission_content_view, permission_document_type_parsing_setup,
    permission_parse_document
)


def is_document_page_disabled(context):
    return not context['resolved_object'].enabled


link_document_content = Link(
    args='resolved_object.id',
    icon_class_path='mayan.apps.document_parsing.icons.icon_document_content',
    permissions=(permission_content_view,), text=_('Content'),
    view='document_parsing:document_content'
)
link_document_content_delete = Link(
    args='resolved_object.id',
    icon_class_path='mayan.apps.document_parsing.icons.icon_document_content_delete',
    permissions=(permission_parse_document,), text=_('Delete parsed content'),
    view='document_parsing:document_content_delete',
)
link_document_content_delete_multiple = Link(
    icon_class_path='mayan.apps.document_parsing.icons.icon_document_content_delete',
    text=_('Delete parsed content'),
    view='document_parsing:document_content_delete_multiple',
)
link_document_page_content = Link(
    args='resolved_object.id', conditional_disable=is_document_page_disabled,
    icon_class_path='mayan.apps.document_parsing.icons.icon_document_content',
    permissions=(permission_content_view,), text=_('Content'),
    view='document_parsing:document_page_content'
)
link_document_parsing_errors_list = Link(
    args='resolved_object.id',
    icon_class_path='mayan.apps.document_parsing.icons.icon_document_parsing_errors_list',
    permissions=(permission_content_view,), text=_('Parsing errors'),
    view='document_parsing:document_parsing_error_list'
)
link_document_content_download = Link(
    args='resolved_object.id',
    icon_class_path='mayan.apps.document_parsing.icons.icon_document_content_download',
    permissions=(permission_content_view,), text=_('Download content'),
    view='document_parsing:document_content_download'
)
link_document_submit_multiple = Link(
    icon_class_path='mayan.apps.document_parsing.icons.icon_document_submit',
    text=_('Submit for parsing'),
    view='document_parsing:document_submit_multiple'
)
link_document_submit = Link(
    args='resolved_object.id',
    icon_class_path='mayan.apps.document_parsing.icons.icon_document_submit',
    permissions=(permission_parse_document,),
    text=_('Submit for parsing'), view='document_parsing:document_submit'
)
link_document_type_parsing_settings = Link(
    args='resolved_object.id',
    icon_class_path='mayan.apps.document_parsing.icons.icon_document_type_parsing_settings',
    permissions=(permission_document_type_parsing_setup,),
    text=_('Setup parsing'),
    view='document_parsing:document_type_parsing_settings'
)
link_document_type_submit = Link(
    condition=get_cascade_condition(
        app_label='documents', model_name='DocumentType',
        object_permission=permission_document_type_parsing_setup
    ),
    icon_class_path='mayan.apps.document_parsing.icons.icon_document_type_submit',
    text=_('Parse documents per type'),
    view='document_parsing:document_type_submit'
)
link_error_list = Link(
    icon_class_path='mayan.apps.document_parsing.icons.icon_link_error_list',
    permissions=(permission_parse_document,), text=_('Parsing errors'),
    view='document_parsing:error_list'
)



from __future__ import unicode_literals

ENVIRONMENT_TEST_NAME = 'PAGINATE_BY'
ENVIRONMENT_TEST_VALUE = '999999'

TEST_BOOTSTAP_SETTING_NAME = 'TEST_BOOTSTRAP_SETTING'

TEST_NAMESPACE_LABEL = 'test namespace label'
TEST_NAMESPACE_NAME = 'test_namespace_name'

TEST_SETTING_GLOBAL_NAME = 'SMART_SETTINGS_TEST_SETTING'
TEST_SETTING_INITIAL_VALUE = 'test_setting_initial_value'
TEST_SETTING_DEFAULT_VALUE = 'test_setting_default_value'
TEST_SETTING_VALUE = 'test_setting_value'
TEST_SETTING_VALUE_OVERRIDE = 'test value override'



from __future__ import unicode_literals

from mayan.apps.common.tests.base import BaseTransactionTestCase
from mayan.apps.documents.tests.literals import TEST_HYBRID_DOCUMENT
from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.document_indexing.models import Index, IndexInstanceNode
from mayan.apps.document_indexing.tests.literals import TEST_INDEX_LABEL

from .literals import TEST_PARSING_INDEX_NODE_TEMPLATE


class ParsingIndexingTestCase(DocumentTestMixin, BaseTransactionTestCase):
    auto_upload_document = False
    test_document_filename = TEST_HYBRID_DOCUMENT

    def test_parsing_indexing(self):
        self.test_index = Index.objects.create(label=TEST_INDEX_LABEL)

        self.test_index.document_types.add(self.test_document_type)

        root = self.test_index.template_root
        self.test_index.node_templates.create(
            parent=root, expression=TEST_PARSING_INDEX_NODE_TEMPLATE,
            link_documents=True
        )

        self.upload_document()
        self.test_document.submit_for_parsing()

        self.assertTrue(
            self.test_document in IndexInstanceNode.objects.get(
                value='sample'
            ).documents.all()
        )



from __future__ import unicode_literals

from rest_framework import serializers

from .models import DocumentPageOCRContent


class DocumentPageOCRContentSerializer(serializers.ModelSerializer):
    class Meta:
        fields = ('content',)
        model = DocumentPageOCRContent



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _


def get_user_label_text(context):
    if not context['request'].user.is_authenticated:
        return _('Anonymous')
    else:
        return context['request'].user.get_full_name() or context['request'].user



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig


class PlatformApp(MayanAppConfig):
    app_namespace = 'platform'
    app_url = 'platform'
    has_rest_api = False
    has_tests = True
    name = 'mayan.apps.platform'
    verbose_name = _('Platform')

    def ready(self):
        super(PlatformApp, self).ready()



from __future__ import unicode_literals

from django.apps import apps
from django.utils.translation import ugettext_lazy as _


def method_get_document_cabinets(self):
    DocumentCabinet = apps.get_model(
        app_label='cabinets', model_name='DocumentCabinet'
    )

    return DocumentCabinet.objects.filter(documents=self).order_by(
        'parent__label', 'label'
    )


method_get_document_cabinets.help_text = _(
    'Return a list of cabinets containing the document'
)
method_get_document_cabinets.short_description = _('get_cabinets()')



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from mayan.apps.acls.models import AccessControlList
from mayan.apps.django_gpg.models import Key
from mayan.apps.django_gpg.permissions import permission_key_sign
from mayan.apps.rest_api.relations import MultiKwargHyperlinkedIdentityField

from .models import DetachedSignature, EmbeddedSignature


class BaseSignatureSerializer(serializers.HyperlinkedModelSerializer):
    document_version_url = MultiKwargHyperlinkedIdentityField(
        view_kwargs=(
            {
                'lookup_field': 'document_version_id',
                'lookup_url_kwarg': 'version_pk',
            },
            {
                'lookup_field': 'document_version.document.pk',
                'lookup_url_kwarg': 'pk',
            }
        ),
        view_name='rest_api:documentversion-detail'
    )


class BaseSignSerializer(serializers.HyperlinkedModelSerializer):
    passphrase = serializers.CharField(
        help_text=_(
            'The passphrase to unlock the key and allow it to be used to '
            'sign the document version.'
        ),
        required=False, write_only=True
    )


class DetachedSignatureSerializer(BaseSignatureSerializer):
    url = MultiKwargHyperlinkedIdentityField(
        view_kwargs=(
            {
                'lookup_field': 'document_version.document.pk',
                'lookup_url_kwarg': 'document_id',
            },
            {
                'lookup_field': 'document_version_id',
                'lookup_url_kwarg': 'document_version_id',
            },
            {
                'lookup_field': 'pk',
                'lookup_url_kwarg': 'detached_signature_id',
            },
        ),
        view_name='rest_api:detachedsignature-detail'
    )

    class Meta:
        extra_kwargs = {
            'signature_file': {'write_only': True},
        }
        fields = (
            'date', 'document_version_url', 'key_id', 'signature_file',
            'signature_id', 'public_key_fingerprint', 'url'
        )
        model = DetachedSignature
        read_only_fields = ('key_id',)

    def create(self, validated_data):
        validated_data['document_version'] = self.context['document_version']
        return super(DetachedSignatureSerializer, self).create(
            validated_data=validated_data
        )


class EmbeddedSignatureSerializer(serializers.HyperlinkedModelSerializer):
    document_version_url = MultiKwargHyperlinkedIdentityField(
        view_kwargs=(
            {
                'lookup_field': 'document_version_id',
                'lookup_url_kwarg': 'version_pk',
            },
            {
                'lookup_field': 'document_version.document.pk',
                'lookup_url_kwarg': 'pk',
            }
        ),
        view_name='rest_api:documentversion-detail'
    )

    url = MultiKwargHyperlinkedIdentityField(
        view_kwargs=(
            {
                'lookup_field': 'document_version.document.pk',
                'lookup_url_kwarg': 'document_id',
            },
            {
                'lookup_field': 'document_version_id',
                'lookup_url_kwarg': 'document_version_id',
            },
            {
                'lookup_field': 'pk',
                'lookup_url_kwarg': 'embedded_signature_id',
            },
        ),
        view_name='rest_api:embeddedsignature-detail'
    )
    passphrase = serializers.CharField(required=False, write_only=True)

    class Meta:
        fields = (
            'date', 'document_version_url', 'key_id', 'signature_id',
            'passphrase', 'public_key_fingerprint', 'url'
        )
        model = EmbeddedSignature

    def create(self, validated_data):
        key_id = validated_data.pop('key_id')
        passphrase = validated_data.pop('passphrase', None)

        key_queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_key_sign, queryset=Key.objects.all(),
            user=self.context['request'].user
        )

        try:
            key = key_queryset.get(fingerprint__endswith=key_id)
        except Key.DoesNotExist:
            raise ValidationError(
                {
                    'key_id': [
                        'Key "{}" not found.'.format(key_id)
                    ]
                }, code='invalid'
            )

        signature = EmbeddedSignature.objects.sign_document_version(
            document_version=self.context['document_version'], key=key,
            passphrase=passphrase, user=self.context['request'].user
        )

        return signature


class SignDetachedSerializer(BaseSignatureSerializer, BaseSignSerializer):
    class Meta:
        fields = (
            'date', 'document_version_url', 'key_id', 'signature_id',
            'passphrase', 'public_key_fingerprint', 'url'
        )
        model = DetachedSignature

    def sign(self, key_id, passphrase):
        key_queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_key_sign, queryset=Key.objects.all(),
            user=self.context['request'].user
        )

        try:
            key = key_queryset.get(fingerprint__endswith=key_id)
        except Key.DoesNotExist:
            raise ValidationError(
                {
                    'key_id': [
                        'Key "{}" not found.'.format(key_id)
                    ]
                }, code='invalid'
            )

        return DetachedSignature.objects.sign_document_version(
            document_version=self.context['document_version'], key=key,
            passphrase=passphrase, user=self.context['request'].user
        )


class SignEmbeddedSerializer(SignDetachedSerializer):
    class Meta:
        fields = (
            'date', 'document_version_url', 'key_id', 'signature_id',
            'passphrase', 'public_key_fingerprint', 'url'
        )
        model = EmbeddedSignature

    def sign(self, key_id, passphrase):
        key_queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_key_sign, queryset=Key.objects.all(),
            user=self.context['request'].user
        )

        try:
            key = key_queryset.get(fingerprint__endswith=key_id)
        except Key.DoesNotExist:
            raise ValidationError(
                {
                    'key_id': [
                        'Key "{}" not found.'.format(key_id)
                    ]
                }, code='invalid'
            )

        return EmbeddedSignature.objects.sign_document_version(
            document_version=self.context['document_version'], key=key,
            passphrase=passphrase, user=self.context['request'].user
        )



from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document_states', '0002_workflowstate_completion'),
    ]

    operations = [
        migrations.CreateModel(
            name='WorkflowRuntimeProxy',
            fields=[
            ],
            options={
                'verbose_name': 'Workflow runtime proxy',
                'proxy': True,
                'verbose_name_plural': 'Workflow runtime proxies',
            },
            bases=('document_states.workflow',),
        ),
        migrations.CreateModel(
            name='WorkflowStateRuntimeProxy',
            fields=[
            ],
            options={
                'verbose_name': 'Workflow state runtime proxy',
                'proxy': True,
                'verbose_name_plural': 'Workflow state runtime proxies',
            },
            bases=('document_states.workflowstate',),
        ),
        migrations.AlterModelOptions(
            name='workflow',
            options={
                'ordering': ('label',),
                'verbose_name': 'Workflow', 'verbose_name_plural': 'Workflows'
            },
        ),
        migrations.AlterModelOptions(
            name='workflowstate',
            options={
                'ordering': ('label',),
                'verbose_name': 'Workflow state',
                'verbose_name_plural': 'Workflow states'
            },
        ),
        migrations.AlterModelOptions(
            name='workflowtransition',
            options={
                'ordering': ('label',), 'verbose_name': 'Workflow transition',
                'verbose_name_plural': 'Workflow transitions'
            },
        ),
    ]



from __future__ import unicode_literals

from django.contrib import messages
from django.http import Http404
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.generics import FormView, SingleObjectListView

from .classes import Namespace, Setting
from .forms import SettingForm
from .permissions import permission_settings_edit, permission_settings_view


class NamespaceDetailView(SingleObjectListView):
    view_permission = permission_settings_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'object': self.get_namespace(),
            'subtitle': _(
                'Settings inherited from an environment variable take '
                'precedence and cannot be changed in this view. '
            ),
            'title': _('Settings in namespace: %s') % self.get_namespace(),
        }

    def get_namespace(self):
        try:
            return Namespace.get(name=self.kwargs['namespace_name'])
        except KeyError:
            raise Http404(
                _('Namespace: %s, not found') % self.kwargs['namespace_name']
            )

    def get_source_queryset(self):
        return self.get_namespace().settings


class NamespaceListView(SingleObjectListView):
    extra_context = {
        'hide_link': True,
        'title': _('Setting namespaces'),
    }
    view_permission = permission_settings_view

    def get_source_queryset(self):
        return Namespace.get_all()


class SettingEditView(FormView):
    form_class = SettingForm
    view_permission = permission_settings_edit

    def form_valid(self, form):
        self.get_object().value = form.cleaned_data['value']
        Setting.save_configuration()
        messages.success(
            message=_('Setting updated successfully.'),
            request=self.request
        )
        return super(SettingEditView, self).form_valid(form=form)

    def get_extra_context(self):
        return {
            'hide_link': True,
            'object': self.get_object(),
            'title': _('Edit setting: %s') % self.get_object(),
        }

    def get_initial(self):
        return {'setting': self.get_object()}

    def get_object(self):
        return Setting.get(self.kwargs['setting_global_name'])

    def get_post_action_redirect(self):
        return reverse(
            viewname='settings:namespace_detail', kwargs={
                'namespace_name': self.get_object().namespace.name
            }
        )



from __future__ import unicode_literals

import platform

if platform.system() in ('FreeBSD', 'OpenBSD', 'Darwin'):
    DEFAULT_EXIF_PATH = '/usr/local/bin/exiftool'
else:
    DEFAULT_EXIF_PATH = '/usr/bin/exiftool'

LOCK_EXPIRE = 60 * 10  # Adjust to worst case scenario



from __future__ import unicode_literals

from django.db import models, migrations
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '__first__'),
    ]

    operations = [
        migrations.CreateModel(
            name='Index',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'name', models.CharField(
                        help_text='Internal name used to reference this '
                        'index.', unique=True, max_length=64,
                        verbose_name='Name'
                    )
                ),
                (
                    'title', models.CharField(
                        help_text='The name that will be visible to users.',
                        unique=True, max_length=128, verbose_name='Title'
                    )
                ),
                (
                    'enabled', models.BooleanField(
                        default=True, help_text='Causes this index to be '
                        'visible and updated when document data changes.',
                        verbose_name='Enabled'
                    )
                ),
                (
                    'document_types',
                    models.ManyToManyField(
                        to='documents.DocumentType',
                        verbose_name='Document types'
                    )
                ),
            ],
            options={
                'verbose_name': 'Index',
                'verbose_name_plural': 'Indexes',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='IndexInstanceNode',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'value', models.CharField(
                        max_length=128, verbose_name='Value', blank=True
                    )
                ),
                (
                    'lft', models.PositiveIntegerField(
                        editable=False, db_index=True
                    )
                ),
                (
                    'rght', models.PositiveIntegerField(
                        editable=False, db_index=True
                    )
                ),
                (
                    'tree_id', models.PositiveIntegerField(
                        editable=False, db_index=True
                    )
                ),
                (
                    'level', models.PositiveIntegerField(
                        editable=False, db_index=True
                    )
                ),
                (
                    'documents', models.ManyToManyField(
                        related_name='node_instances',
                        verbose_name='Documents', to='documents.Document'
                    )
                ),
            ],
            options={
                'verbose_name': 'Index node instance',
                'verbose_name_plural': 'Indexes node instances',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='IndexTemplateNode',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'expression', models.CharField(
                        help_text='Enter a python string expression to be '
                        'evaluated.', max_length=128,
                        verbose_name='Indexing expression'
                    )
                ),
                (
                    'enabled', models.BooleanField(
                        default=True, help_text='Causes this node to be '
                        'visible and updated when document data changes.',
                        verbose_name='Enabled'
                    )
                ),
                (
                    'link_documents', models.BooleanField(
                        default=False, help_text='Check this option to have '
                        'this node act as a container for documents and not '
                        'as a parent for further nodes.',
                        verbose_name='Link documents'
                    )
                ),
                (
                    'lft', models.PositiveIntegerField(
                        editable=False, db_index=True
                    )
                ),
                (
                    'rght', models.PositiveIntegerField(
                        editable=False, db_index=True
                    )
                ),
                (
                    'tree_id', models.PositiveIntegerField(
                        editable=False, db_index=True
                    )
                ),
                (
                    'level', models.PositiveIntegerField(
                        editable=False, db_index=True
                    )
                ),
                (
                    'index', models.ForeignKey(
                        related_name='node_templates', verbose_name='Index',
                        to='document_indexing.Index'
                    )
                ),
                (
                    'parent', mptt.fields.TreeForeignKey(
                        blank=True, to='document_indexing.IndexTemplateNode',
                        null=True
                    )
                ),
            ],
            options={
                'verbose_name': 'Index node template',
                'verbose_name_plural': 'Indexes node template',
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='indexinstancenode',
            name='index_template_node',
            field=models.ForeignKey(
                related_name='node_instance',
                verbose_name='Index template node',
                to='document_indexing.IndexTemplateNode'
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='indexinstancenode',
            name='parent',
            field=mptt.fields.TreeForeignKey(
                blank=True, to='document_indexing.IndexInstanceNode',
                null=True
            ),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals

import os

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

from .setting_migrations import SourcesSettingMigration

namespace = Namespace(
    label=_('Sources'), migration_class=SourcesSettingMigration,
    name='sources', version='0002'
)

setting_scanimage_path = namespace.add_setting(
    global_name='SOURCES_SCANIMAGE_PATH', default='/usr/bin/scanimage',
    help_text=_(
        'File path to the scanimage program used to control image scanners.'
    ),
    is_path=True
)
setting_staging_file_image_cache_storage = namespace.add_setting(
    global_name='SOURCES_STAGING_FILE_CACHE_STORAGE_BACKEND',
    default='django.core.files.storage.FileSystemStorage', help_text=_(
        'Path to the Storage subclass to use when storing the cached '
        'staging_file image files.'
    )
)
setting_staging_file_image_cache_storage_arguments = namespace.add_setting(
    global_name='SOURCES_STAGING_FILE_CACHE_STORAGE_BACKEND_ARGUMENTS',
    default={
        'location': os.path.join(settings.MEDIA_ROOT, 'staging_file_cache')
    }, help_text=_(
        'Arguments to pass to the SOURCES_STAGING_FILE_CACHE_STORAGE_BACKEND.'
    )
)



from __future__ import unicode_literals


class FakeStorageSubclass(object):
    """
    Placeholder class to allow serializing the real storage subclass to
    support migrations.
    """
    def __eq__(self, other):
        return True



from __future__ import unicode_literals

import logging

from django.apps import apps
from django.db import OperationalError

from mayan.apps.lock_manager.exceptions import LockError
from mayan.celery import app

from .literals import RETRY_DELAY

logger = logging.getLogger(__name__)


@app.task(bind=True, default_retry_delay=RETRY_DELAY, max_retries=None, ignore_result=True)
def task_delete_empty(self):
    IndexInstanceNode = apps.get_model(
        app_label='document_indexing', model_name='IndexInstanceNode'
    )

    try:
        IndexInstanceNode.objects.delete_empty()
    except LockError as exception:
        raise self.retry(exc=exception)


@app.task(bind=True, default_retry_delay=RETRY_DELAY, max_retries=None, ignore_result=True)
def task_index_document(self, document_id):
    Document = apps.get_model(
        app_label='documents', model_name='Document'
    )
    Index = apps.get_model(
        app_label='document_indexing', model_name='Index'
    )

    try:
        document = Document.objects.get(pk=document_id)
    except Document.DoesNotExist:
        # Document was deleted before we could execute, abort about
        # updating
        pass
    else:
        try:
            Index.objects.index_document(document=document)
        except OperationalError as exception:
            logger.warning(
                'Operational error while trying to index document: '
                '%s; %s', document, exception
            )
            raise self.retry(exc=exception)
        except LockError as exception:
            logger.warning(
                'Unable to acquire lock for document %s; %s ',
                document, exception
            )
            raise self.retry(exc=exception)


@app.task(bind=True, default_retry_delay=RETRY_DELAY, ignore_result=True)
def task_rebuild_index(self, index_id):
    Index = apps.get_model(
        app_label='document_indexing', model_name='Index'
    )

    try:
        index = Index.objects.get(pk=index_id)
        index.rebuild()
    except LockError as exception:
        # This index is being rebuilt by another task, retry later
        raise self.retry(exc=exception)


@app.task(bind=True, default_retry_delay=RETRY_DELAY, max_retries=None, ignore_result=True)
def task_remove_document(self, document_id):
    Document = apps.get_model(
        app_label='documents', model_name='Document'
    )
    IndexInstanceNode = apps.get_model(
        app_label='document_indexing', model_name='IndexInstanceNode'
    )

    try:
        document = Document.objects.get(pk=document_id)
    except Document.DoesNotExist:
        # Document was deleted before we could execute
        # Since it was automatically removed from the document M2M
        # we just now delete the empty instance nodes
        try:
            IndexInstanceNode.objects.delete_empty()
        except LockError as exception:
            raise self.retry(exc=exception)
    else:
        try:
            IndexInstanceNode.objects.remove_document(document=document)
        except LockError as exception:
            raise self.retry(exc=exception)



from __future__ import absolute_import, unicode_literals

import logging

from django.utils.translation import ugettext_lazy as _, ungettext

from mayan.apps.common.exceptions import ActionError
from mayan.apps.common.generics import MultipleObjectConfirmActionView

from ..icons import icon_favorite_document_list
from ..models import Document, FavoriteDocument
from ..permissions import permission_document_view
from ..settings import setting_favorite_count

from .document_views import DocumentListView

__all__ = (
    'FavoriteDocumentListView', 'FavoriteAddView', 'FavoriteRemoveView'
)
logger = logging.getLogger(__name__)


class FavoriteDocumentListView(DocumentListView):
    def get_document_queryset(self):
        return FavoriteDocument.objects.get_for_user(user=self.request.user)

    def get_extra_context(self):
        context = super(FavoriteDocumentListView, self).get_extra_context()
        context.update(
            {
                'no_results_icon': icon_favorite_document_list,
                'no_results_text': _(
                    'Favorited documents will be listed in this view. '
                    'Up to %(count)d documents can be favorited per user. '
                ) % {'count': setting_favorite_count.value},
                'no_results_title': _('There are no favorited documents.'),
                'title': _('Favorites'),
            }
        )
        return context


class FavoriteAddView(MultipleObjectConfirmActionView):
    model = Document
    object_permission = permission_document_view
    success_message = _(
        '%(count)d document added to favorites.'
    )
    success_message_plural = _(
        '%(count)d documents added to favorites.'
    )

    def get_extra_context(self):
        queryset = self.object_list

        return {
            'submit_label': _('Add'),
            'submit_icon_class': icon_favorite_document_list,
            'title': ungettext(
                singular='Add the selected document to favorites',
                plural='Add the selected documents to favorites',
                number=queryset.count()
            )
        }

    def object_action(self, form, instance):
        FavoriteDocument.objects.add_for_user(
            user=self.request.user, document=instance
        )


class FavoriteRemoveView(MultipleObjectConfirmActionView):
    error_message = _('Document "%(instance)s" is not in favorites.')
    model = Document
    object_permission = permission_document_view
    success_message = _(
        '%(count)d document removed from favorites.'
    )
    success_message_plural = _(
        '%(count)d documents removed from favorites.'
    )

    def get_extra_context(self):
        queryset = self.object_list

        return {
            'submit_label': _('Remove'),
            'submit_icon_class': icon_favorite_document_list,
            'title': ungettext(
                singular='Remove the selected document from favorites',
                plural='Remove the selected documents from favorites',
                number=queryset.count()
            )
        }

    def object_action(self, form, instance):
        try:
            FavoriteDocument.objects.remove_for_user(
                user=self.request.user, document=instance
            )
        except FavoriteDocument.DoesNotExist:
            raise ActionError



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document_indexing', '0008_auto_20150729_1515'),
    ]

    operations = [
        migrations.AlterField(
            model_name='indextemplatenode',
            name='expression',
            field=models.CharField(
                help_text="Enter a template to render. Use Django's default "
                "templating language (https://docs.djangoproject.com/en/1.7/"
                "ref/templates/builtins/)", max_length=128,
                verbose_name='Indexing expression'
            ),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals

import json

from django import forms
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.common.forms import DynamicModelForm
from mayan.apps.common.settings import setting_project_title, setting_project_url

from .classes import MailerBackend
from .models import UserMailer
from .permissions import permission_user_mailer_use
from .settings import (
    setting_document_body_template, setting_document_subject_template,
    setting_link_body_template, setting_link_subject_template
)
from .validators import validate_email_multiple


class DocumentMailForm(forms.Form):
    def __init__(self, *args, **kwargs):
        as_attachment = kwargs.pop('as_attachment', False)
        user = kwargs.pop('user', None)
        super(DocumentMailForm, self).__init__(*args, **kwargs)
        if as_attachment:
            self.fields[
                'subject'
            ].initial = setting_document_subject_template.value

            self.fields[
                'body'
            ].initial = setting_document_body_template.value % {
                'project_title': setting_project_title.value,
                'project_website': setting_project_url.value
            }
        else:
            self.fields[
                'subject'
            ].initial = setting_link_subject_template.value
            self.fields['body'].initial = setting_link_body_template.value % {
                'project_title': setting_project_title.value,
                'project_website': setting_project_url.value
            }

        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_user_mailer_use, user=user,
            queryset=UserMailer.objects.filter(enabled=True)
        )

        self.fields['user_mailer'].queryset = queryset
        try:
            self.fields['user_mailer'].initial = queryset.get(default=True)
        except UserMailer.DoesNotExist:
            pass

    email = forms.CharField(
        help_text=_(
            'Email address of the recipient. Can be multiple addresses '
            'separated by comma or semicolon.'
        ), label=_('Email address'), validators=[validate_email_multiple]
    )
    subject = forms.CharField(label=_('Subject'), required=False)
    body = forms.CharField(
        label=_('Body'), widget=forms.widgets.Textarea(), required=False
    )
    user_mailer = forms.ModelChoiceField(
        help_text=_(
            'The email profile that will be used to send this email.'
        ), label=_('Mailing profile'), queryset=UserMailer.objects.none()
    )


class UserMailerBackendSelectionForm(forms.Form):
    backend = forms.ChoiceField(
        choices=(), help_text=_('The driver to use when sending emails.'),
        label=_('Backend')
    )

    def __init__(self, *args, **kwargs):
        super(UserMailerBackendSelectionForm, self).__init__(*args, **kwargs)
        self.fields['backend'].choices = [
            (
                key, backend.label
            ) for key, backend in MailerBackend.get_all().items()
        ]


class UserMailerDynamicForm(DynamicModelForm):
    class Meta:
        fields = ('label', 'default', 'enabled', 'backend_data')
        model = UserMailer
        widgets = {'backend_data': forms.widgets.HiddenInput}

    def __init__(self, *args, **kwargs):
        result = super(UserMailerDynamicForm, self).__init__(*args, **kwargs)
        if self.instance.backend_data:
            backend_data = json.loads(self.instance.backend_data)
            for key in self.instance.get_backend().fields:
                self.fields[key].initial = backend_data.get(key)

        return result

    def clean(self):
        data = super(UserMailerDynamicForm, self).clean()

        # Consolidate the dynamic fields into a single JSON field called
        # 'backend_data'.
        backend_data = {}

        for field_name, field_data in self.schema['fields'].items():
            backend_data[field_name] = data.pop(
                field_name, field_data.get('default', None)
            )

        data['backend_data'] = json.dumps(backend_data)
        return data


class UserMailerTestForm(forms.Form):
    email = forms.CharField(
        help_text=_(
            'Email address of the recipient. Can be multiple addresses '
            'separated by comma or semicolon.'
        ), label=_('Email address'), validators=[validate_email_multiple]
    )



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig


class AppearanceApp(MayanAppConfig):
    has_tests = True
    name = 'mayan.apps.appearance'
    verbose_name = _('Appearance')

    def ready(self):
        super(AppearanceApp, self).ready()



from __future__ import unicode_literals

from actstream.models import Action

from ..events import (
    event_document_download, event_document_trashed, event_document_view
)
from ..permissions import (
    permission_document_download, permission_document_trash,
    permission_document_view
)

from .base import GenericDocumentViewTestCase
from .mixins import TrashedDocumentViewTestMixin

TEST_DOCUMENT_TYPE_EDITED_LABEL = 'test document type edited label'
TEST_DOCUMENT_TYPE_2_LABEL = 'test document type 2 label'
TEST_TRANSFORMATION_NAME = 'rotate'
TEST_TRANSFORMATION_ARGUMENT = 'degrees: 180'


class DocumentEventsTestMixin(object):
    def _request_test_document_download_view(self):
        return self.get(
            'documents:document_download', kwargs={'pk': self.test_document.pk}
        )

    def _request_test_document_preview_view(self):
        return self.get(
            viewname='documents:document_preview', kwargs={
                'pk': self.test_document.pk
            }
        )


class DocumentEventsTestCase(
    DocumentEventsTestMixin, TrashedDocumentViewTestMixin,
    GenericDocumentViewTestCase
):
    def setUp(self):
        super(DocumentEventsTestCase, self).setUp()
        Action.objects.all().delete()

    def test_document_download_event_no_permission(self):
        response = self._request_test_document_download_view()

        self.assertEqual(response.status_code, 404)
        self.assertEqual(list(Action.objects.any(obj=self.test_document)), [])

    def test_document_download_event_with_access(self):
        self.expected_content_types = ('image/png',)

        self.grant_access(
            obj=self.test_document, permission=permission_document_download
        )

        response = self._request_test_document_download_view()

        # Download the file to close the file descriptor
        with self.test_document.open() as file_object:
            self.assert_download_response(
                response=response, content=file_object.read(),
                mime_type=self.test_document.file_mimetype
            )

        event = Action.objects.any(obj=self.test_document).first()

        self.assertEqual(event.actor, self._test_case_user)
        self.assertEqual(event.target, self.test_document)
        self.assertEqual(event.verb, event_document_download.id)

    def test_document_view_event_no_permission(self):
        response = self._request_test_document_preview_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(list(Action.objects.any(obj=self.test_document)), [])

    def test_document_view_event_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_preview_view()
        self.assertEqual(response.status_code, 200)

        event = Action.objects.any(obj=self.test_document).first()
        self.assertEqual(event.actor, self._test_case_user)
        self.assertEqual(event.target, self.test_document)
        self.assertEqual(event.verb, event_document_view.id)

    def test_document_trashed_view_event_no_permission(self):
        response = self._request_document_trash_post_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(list(Action.objects.any(obj=self.test_document)), [])

    def test_document_trashed_view_event_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_trash
        )

        response = self._request_document_trash_post_view()
        self.assertEqual(response.status_code, 302)

        event = Action.objects.any(obj=self.test_document).first()
        self.assertEqual(event.actor, self._test_case_user)
        self.assertEqual(event.target, self.test_document)
        self.assertEqual(event.verb, event_document_trashed.id)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

from .literals import (
    DEFAULT_LIBREOFFICE_PATH, DEFAULT_PDFTOPPM_DPI, DEFAULT_PDFTOPPM_FORMAT,
    DEFAULT_PDFTOPPM_PATH, DEFAULT_PDFINFO_PATH, DEFAULT_PILLOW_FORMAT,
    DEFAULT_PILLOW_MAXIMUM_IMAGE_PIXELS
)
from .setting_migrations import ConvertSettingMigration

namespace = Namespace(
    label=_('Converter'), migration_class=ConvertSettingMigration,
    name='converter', version='0002'
)

setting_graphics_backend = namespace.add_setting(
    default='mayan.apps.converter.backends.python.Python',
    help_text=_('Graphics conversion backend to use.'),
    global_name='CONVERTER_GRAPHICS_BACKEND',
)
setting_graphics_backend_arguments = namespace.add_setting(
    default={
        'libreoffice_path': DEFAULT_LIBREOFFICE_PATH,
        'pdftoppm_dpi': DEFAULT_PDFTOPPM_DPI,
        'pdftoppm_format': DEFAULT_PDFTOPPM_FORMAT,
        'pdftoppm_path': DEFAULT_PDFTOPPM_PATH,
        'pdfinfo_path': DEFAULT_PDFINFO_PATH,
        'pillow_format': DEFAULT_PILLOW_FORMAT,
        'pillow_maximum_image_pixels': DEFAULT_PILLOW_MAXIMUM_IMAGE_PIXELS,
    }, help_text=_(
        'Configuration options for the graphics conversion backend.'
    ), global_name='CONVERTER_GRAPHICS_BACKEND_ARGUMENTS'
)



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('motd', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messageoftheday',
            name='end_datetime',
            field=models.DateTimeField(
                null=True, verbose_name='End date time', blank=True
            ),
        ),
        migrations.AlterField(
            model_name='messageoftheday',
            name='start_datetime',
            field=models.DateTimeField(
                null=True, verbose_name='Start date time', blank=True
            ),
        ),
    ]



from __future__ import unicode_literals

from mayan.apps.common.tests.base import BaseTestCase

from .mixins import WorkflowTestMixin


class WorkflowModelTestCase(WorkflowTestMixin, BaseTestCase):
    def test_workflow_template_preview(self):
        self._create_test_workflow()
        self.assertTrue(self.test_workflow.get_api_image_url())



from __future__ import unicode_literals

from django.conf.urls import url

from .api_views import (
    APIAdvancedSearchView, APISearchModelList, APISearchView
)
from .views import (
    AdvancedSearchView, ResultsView, SearchAgainView, SearchView
)

urlpatterns = [
    url(
        regex=r'^search/(?P<search_model>[\.\w]+)/$', view=SearchView.as_view(),
        name='search'
    ),
    url(
        regex=r'^advanced/(?P<search_model>[\.\w]+)/$',
        view=AdvancedSearchView.as_view(), name='search_advanced'
    ),
    url(
        regex=r'^advanced/$',
        view=AdvancedSearchView.as_view(), name='search_advanced'
    ),
    url(
        regex=r'^again/(?P<search_model>[\.\w]+)/$',
        view=SearchAgainView.as_view(), name='search_again'
    ),
    url(
        regex=r'^results/$',
        view=ResultsView.as_view(), name='results'
    ),
    url(
        regex=r'^results/(?P<search_model>[\.\w]+)/$',
        view=ResultsView.as_view(), name='results'
    ),
]

api_urls = [
    url(
        regex=r'^search_models/$', view=APISearchModelList.as_view(),
        name='searchmodel-list'
    ),
    url(
        regex=r'^search/(?P<search_model>[\.\w]+)/$',
        view=APISearchView.as_view(), name='search-view'
    ),
    url(
        regex=r'^search/advanced/(?P<search_model>[\.\w]+)/$',
        view=APIAdvancedSearchView.as_view(), name='advanced-search-view'
    ),
]



from __future__ import print_function, unicode_literals

import mock

import mayan

from mayan.apps.common.tests.base import BaseTestCase

from ..utils import PyPIClient


class PyPIClientTestCase(BaseTestCase):
    @mock.patch('mayan.apps.dependencies.utils.PyPIClient.get_versions', autospec=True)
    def test_check_version_not_latest_version(self, mock_package_releases):
        mock_package_releases.return_value = ('0.0.0',)
        with self.assertRaises(PyPIClient.NotLatestVersion):
            PyPIClient().check_version()

    @mock.patch('mayan.apps.dependencies.utils.PyPIClient.get_versions', autospec=True)
    def test_check_version_unknown_version(self, mock_package_releases):
        mock_package_releases.return_value = None
        with self.assertRaises(PyPIClient.UnknownLatestVersion):
            PyPIClient().check_version()

    @mock.patch('mayan.apps.dependencies.utils.PyPIClient.get_versions', autospec=True)
    def test_check_version_correct_version(self, mock_package_releases):
        mock_package_releases.return_value = (mayan.__version__,)
        PyPIClient().check_version()



from __future__ import unicode_literals

from django.template.loader import render_to_string


class SourceColumnLinkWidget(object):
    template_name = 'navigation/source_column_link_widget.html'

    def render(self, name=None, value=None):
        return render_to_string(
            template_name=self.template_name, context={
                'column': self.column, 'value': value
            }
        )



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

from .literals import DEFAULT_MAXIMUM_TITLE_LENGTH, DEFAULT_MESSAGE_POSITION

namespace = Namespace(label=_('Appearance'), name='appearance')

setting_max_title_length = namespace.add_setting(
    default=DEFAULT_MAXIMUM_TITLE_LENGTH,
    global_name='APPEARANCE_MAXIMUM_TITLE_LENGTH', help_text=_(
        'Maximum number of characters that will be displayed as the view '
        'title.'
    )
)
setting_message_position = namespace.add_setting(
    default=DEFAULT_MESSAGE_POSITION,
    global_name='APPEARANCE_MESSAGE_POSITION', help_text=_(
        'Position where the system message will be displayed. Options are: '
        'top-left, top-center, top-right, bottom-left, bottom-center, '
        'bottom-right.'
    )
)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.22 on 2019-07-11 05:48
from __future__ import unicode_literals

from django.db import migrations, models
import mayan.apps.common.models
import mayan.apps.storage.classes


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0011_auto_20180429_0758'),
    ]

    operations = [
        migrations.AlterField(
            model_name='shareduploadedfile',
            name='file',
            field=models.FileField(storage=mayan.apps.storage.classes.FakeStorageSubclass(), upload_to=mayan.apps.common.models.upload_to, verbose_name='File'),
        ),
        migrations.AlterField(
            model_name='userlocaleprofile',
            name='language',
            field=models.CharField(choices=[('ar', 'Arabic'), ('bg', 'Bulgarian'), ('bs', 'Bosnian'), ('cs', 'Czech'), ('da', 'Danish'), ('de', 'German'), ('el', 'Greek'), ('en', 'English'), ('es', 'Spanish'), ('fa', 'Persian'), ('fr', 'French'), ('hu', 'Hungarian'), ('id', 'Indonesian'), ('it', 'Italian'), ('lv', 'Latvian'), ('nl', 'Dutch'), ('pl', 'Polish'), ('pt', 'Portuguese'), ('pt-br', 'Portuguese (Brazil)'), ('ro', 'Romanian'), ('ru', 'Russian'), ('sl', 'Slovenian'), ('tr', 'Turkish'), ('vi', 'Vietnamese'), ('zh', 'Chinese')], max_length=8, verbose_name='Language'),
        ),
        migrations.AlterField(
            model_name='userlocaleprofile',
            name='timezone',
            field=models.CharField(choices=[(b'Africa/Abidjan', b'Africa/Abidjan'), (b'Africa/Accra', b'Africa/Accra'), (b'Africa/Addis_Ababa', b'Africa/Addis_Ababa'), (b'Africa/Algiers', b'Africa/Algiers'), (b'Africa/Asmara', b'Africa/Asmara'), (b'Africa/Bamako', b'Africa/Bamako'), (b'Africa/Bangui', b'Africa/Bangui'), (b'Africa/Banjul', b'Africa/Banjul'), (b'Africa/Bissau', b'Africa/Bissau'), (b'Africa/Blantyre', b'Africa/Blantyre'), (b'Africa/Brazzaville', b'Africa/Brazzaville'), (b'Africa/Bujumbura', b'Africa/Bujumbura'), (b'Africa/Cairo', b'Africa/Cairo'), (b'Africa/Casablanca', b'Africa/Casablanca'), (b'Africa/Ceuta', b'Africa/Ceuta'), (b'Africa/Conakry', b'Africa/Conakry'), (b'Africa/Dakar', b'Africa/Dakar'), (b'Africa/Dar_es_Salaam', b'Africa/Dar_es_Salaam'), (b'Africa/Djibouti', b'Africa/Djibouti'), (b'Africa/Douala', b'Africa/Douala'), (b'Africa/El_Aaiun', b'Africa/El_Aaiun'), (b'Africa/Freetown', b'Africa/Freetown'), (b'Africa/Gaborone', b'Africa/Gaborone'), (b'Africa/Harare', b'Africa/Harare'), (b'Africa/Johannesburg', b'Africa/Johannesburg'), (b'Africa/Juba', b'Africa/Juba'), (b'Africa/Kampala', b'Africa/Kampala'), (b'Africa/Khartoum', b'Africa/Khartoum'), (b'Africa/Kigali', b'Africa/Kigali'), (b'Africa/Kinshasa', b'Africa/Kinshasa'), (b'Africa/Lagos', b'Africa/Lagos'), (b'Africa/Libreville', b'Africa/Libreville'), (b'Africa/Lome', b'Africa/Lome'), (b'Africa/Luanda', b'Africa/Luanda'), (b'Africa/Lubumbashi', b'Africa/Lubumbashi'), (b'Africa/Lusaka', b'Africa/Lusaka'), (b'Africa/Malabo', b'Africa/Malabo'), (b'Africa/Maputo', b'Africa/Maputo'), (b'Africa/Maseru', b'Africa/Maseru'), (b'Africa/Mbabane', b'Africa/Mbabane'), (b'Africa/Mogadishu', b'Africa/Mogadishu'), (b'Africa/Monrovia', b'Africa/Monrovia'), (b'Africa/Nairobi', b'Africa/Nairobi'), (b'Africa/Ndjamena', b'Africa/Ndjamena'), (b'Africa/Niamey', b'Africa/Niamey'), (b'Africa/Nouakchott', b'Africa/Nouakchott'), (b'Africa/Ouagadougou', b'Africa/Ouagadougou'), (b'Africa/Porto-Novo', b'Africa/Porto-Novo'), (b'Africa/Sao_Tome', b'Africa/Sao_Tome'), (b'Africa/Tripoli', b'Africa/Tripoli'), (b'Africa/Tunis', b'Africa/Tunis'), (b'Africa/Windhoek', b'Africa/Windhoek'), (b'America/Adak', b'America/Adak'), (b'America/Anchorage', b'America/Anchorage'), (b'America/Anguilla', b'America/Anguilla'), (b'America/Antigua', b'America/Antigua'), (b'America/Araguaina', b'America/Araguaina'), (b'America/Argentina/Buenos_Aires', b'America/Argentina/Buenos_Aires'), (b'America/Argentina/Catamarca', b'America/Argentina/Catamarca'), (b'America/Argentina/Cordoba', b'America/Argentina/Cordoba'), (b'America/Argentina/Jujuy', b'America/Argentina/Jujuy'), (b'America/Argentina/La_Rioja', b'America/Argentina/La_Rioja'), (b'America/Argentina/Mendoza', b'America/Argentina/Mendoza'), (b'America/Argentina/Rio_Gallegos', b'America/Argentina/Rio_Gallegos'), (b'America/Argentina/Salta', b'America/Argentina/Salta'), (b'America/Argentina/San_Juan', b'America/Argentina/San_Juan'), (b'America/Argentina/San_Luis', b'America/Argentina/San_Luis'), (b'America/Argentina/Tucuman', b'America/Argentina/Tucuman'), (b'America/Argentina/Ushuaia', b'America/Argentina/Ushuaia'), (b'America/Aruba', b'America/Aruba'), (b'America/Asuncion', b'America/Asuncion'), (b'America/Atikokan', b'America/Atikokan'), (b'America/Bahia', b'America/Bahia'), (b'America/Bahia_Banderas', b'America/Bahia_Banderas'), (b'America/Barbados', b'America/Barbados'), (b'America/Belem', b'America/Belem'), (b'America/Belize', b'America/Belize'), (b'America/Blanc-Sablon', b'America/Blanc-Sablon'), (b'America/Boa_Vista', b'America/Boa_Vista'), (b'America/Bogota', b'America/Bogota'), (b'America/Boise', b'America/Boise'), (b'America/Cambridge_Bay', b'America/Cambridge_Bay'), (b'America/Campo_Grande', b'America/Campo_Grande'), (b'America/Cancun', b'America/Cancun'), (b'America/Caracas', b'America/Caracas'), (b'America/Cayenne', b'America/Cayenne'), (b'America/Cayman', b'America/Cayman'), (b'America/Chicago', b'America/Chicago'), (b'America/Chihuahua', b'America/Chihuahua'), (b'America/Costa_Rica', b'America/Costa_Rica'), (b'America/Creston', b'America/Creston'), (b'America/Cuiaba', b'America/Cuiaba'), (b'America/Curacao', b'America/Curacao'), (b'America/Danmarkshavn', b'America/Danmarkshavn'), (b'America/Dawson', b'America/Dawson'), (b'America/Dawson_Creek', b'America/Dawson_Creek'), (b'America/Denver', b'America/Denver'), (b'America/Detroit', b'America/Detroit'), (b'America/Dominica', b'America/Dominica'), (b'America/Edmonton', b'America/Edmonton'), (b'America/Eirunepe', b'America/Eirunepe'), (b'America/El_Salvador', b'America/El_Salvador'), (b'America/Fort_Nelson', b'America/Fort_Nelson'), (b'America/Fortaleza', b'America/Fortaleza'), (b'America/Glace_Bay', b'America/Glace_Bay'), (b'America/Godthab', b'America/Godthab'), (b'America/Goose_Bay', b'America/Goose_Bay'), (b'America/Grand_Turk', b'America/Grand_Turk'), (b'America/Grenada', b'America/Grenada'), (b'America/Guadeloupe', b'America/Guadeloupe'), (b'America/Guatemala', b'America/Guatemala'), (b'America/Guayaquil', b'America/Guayaquil'), (b'America/Guyana', b'America/Guyana'), (b'America/Halifax', b'America/Halifax'), (b'America/Havana', b'America/Havana'), (b'America/Hermosillo', b'America/Hermosillo'), (b'America/Indiana/Indianapolis', b'America/Indiana/Indianapolis'), (b'America/Indiana/Knox', b'America/Indiana/Knox'), (b'America/Indiana/Marengo', b'America/Indiana/Marengo'), (b'America/Indiana/Petersburg', b'America/Indiana/Petersburg'), (b'America/Indiana/Tell_City', b'America/Indiana/Tell_City'), (b'America/Indiana/Vevay', b'America/Indiana/Vevay'), (b'America/Indiana/Vincennes', b'America/Indiana/Vincennes'), (b'America/Indiana/Winamac', b'America/Indiana/Winamac'), (b'America/Inuvik', b'America/Inuvik'), (b'America/Iqaluit', b'America/Iqaluit'), (b'America/Jamaica', b'America/Jamaica'), (b'America/Juneau', b'America/Juneau'), (b'America/Kentucky/Louisville', b'America/Kentucky/Louisville'), (b'America/Kentucky/Monticello', b'America/Kentucky/Monticello'), (b'America/Kralendijk', b'America/Kralendijk'), (b'America/La_Paz', b'America/La_Paz'), (b'America/Lima', b'America/Lima'), (b'America/Los_Angeles', b'America/Los_Angeles'), (b'America/Lower_Princes', b'America/Lower_Princes'), (b'America/Maceio', b'America/Maceio'), (b'America/Managua', b'America/Managua'), (b'America/Manaus', b'America/Manaus'), (b'America/Marigot', b'America/Marigot'), (b'America/Martinique', b'America/Martinique'), (b'America/Matamoros', b'America/Matamoros'), (b'America/Mazatlan', b'America/Mazatlan'), (b'America/Menominee', b'America/Menominee'), (b'America/Merida', b'America/Merida'), (b'America/Metlakatla', b'America/Metlakatla'), (b'America/Mexico_City', b'America/Mexico_City'), (b'America/Miquelon', b'America/Miquelon'), (b'America/Moncton', b'America/Moncton'), (b'America/Monterrey', b'America/Monterrey'), (b'America/Montevideo', b'America/Montevideo'), (b'America/Montserrat', b'America/Montserrat'), (b'America/Nassau', b'America/Nassau'), (b'America/New_York', b'America/New_York'), (b'America/Nipigon', b'America/Nipigon'), (b'America/Nome', b'America/Nome'), (b'America/Noronha', b'America/Noronha'), (b'America/North_Dakota/Beulah', b'America/North_Dakota/Beulah'), (b'America/North_Dakota/Center', b'America/North_Dakota/Center'), (b'America/North_Dakota/New_Salem', b'America/North_Dakota/New_Salem'), (b'America/Ojinaga', b'America/Ojinaga'), (b'America/Panama', b'America/Panama'), (b'America/Pangnirtung', b'America/Pangnirtung'), (b'America/Paramaribo', b'America/Paramaribo'), (b'America/Phoenix', b'America/Phoenix'), (b'America/Port-au-Prince', b'America/Port-au-Prince'), (b'America/Port_of_Spain', b'America/Port_of_Spain'), (b'America/Porto_Velho', b'America/Porto_Velho'), (b'America/Puerto_Rico', b'America/Puerto_Rico'), (b'America/Punta_Arenas', b'America/Punta_Arenas'), (b'America/Rainy_River', b'America/Rainy_River'), (b'America/Rankin_Inlet', b'America/Rankin_Inlet'), (b'America/Recife', b'America/Recife'), (b'America/Regina', b'America/Regina'), (b'America/Resolute', b'America/Resolute'), (b'America/Rio_Branco', b'America/Rio_Branco'), (b'America/Santarem', b'America/Santarem'), (b'America/Santiago', b'America/Santiago'), (b'America/Santo_Domingo', b'America/Santo_Domingo'), (b'America/Sao_Paulo', b'America/Sao_Paulo'), (b'America/Scoresbysund', b'America/Scoresbysund'), (b'America/Sitka', b'America/Sitka'), (b'America/St_Barthelemy', b'America/St_Barthelemy'), (b'America/St_Johns', b'America/St_Johns'), (b'America/St_Kitts', b'America/St_Kitts'), (b'America/St_Lucia', b'America/St_Lucia'), (b'America/St_Thomas', b'America/St_Thomas'), (b'America/St_Vincent', b'America/St_Vincent'), (b'America/Swift_Current', b'America/Swift_Current'), (b'America/Tegucigalpa', b'America/Tegucigalpa'), (b'America/Thule', b'America/Thule'), (b'America/Thunder_Bay', b'America/Thunder_Bay'), (b'America/Tijuana', b'America/Tijuana'), (b'America/Toronto', b'America/Toronto'), (b'America/Tortola', b'America/Tortola'), (b'America/Vancouver', b'America/Vancouver'), (b'America/Whitehorse', b'America/Whitehorse'), (b'America/Winnipeg', b'America/Winnipeg'), (b'America/Yakutat', b'America/Yakutat'), (b'America/Yellowknife', b'America/Yellowknife'), (b'Antarctica/Casey', b'Antarctica/Casey'), (b'Antarctica/Davis', b'Antarctica/Davis'), (b'Antarctica/DumontDUrville', b'Antarctica/DumontDUrville'), (b'Antarctica/Macquarie', b'Antarctica/Macquarie'), (b'Antarctica/Mawson', b'Antarctica/Mawson'), (b'Antarctica/McMurdo', b'Antarctica/McMurdo'), (b'Antarctica/Palmer', b'Antarctica/Palmer'), (b'Antarctica/Rothera', b'Antarctica/Rothera'), (b'Antarctica/Syowa', b'Antarctica/Syowa'), (b'Antarctica/Troll', b'Antarctica/Troll'), (b'Antarctica/Vostok', b'Antarctica/Vostok'), (b'Arctic/Longyearbyen', b'Arctic/Longyearbyen'), (b'Asia/Aden', b'Asia/Aden'), (b'Asia/Almaty', b'Asia/Almaty'), (b'Asia/Amman', b'Asia/Amman'), (b'Asia/Anadyr', b'Asia/Anadyr'), (b'Asia/Aqtau', b'Asia/Aqtau'), (b'Asia/Aqtobe', b'Asia/Aqtobe'), (b'Asia/Ashgabat', b'Asia/Ashgabat'), (b'Asia/Atyrau', b'Asia/Atyrau'), (b'Asia/Baghdad', b'Asia/Baghdad'), (b'Asia/Bahrain', b'Asia/Bahrain'), (b'Asia/Baku', b'Asia/Baku'), (b'Asia/Bangkok', b'Asia/Bangkok'), (b'Asia/Barnaul', b'Asia/Barnaul'), (b'Asia/Beirut', b'Asia/Beirut'), (b'Asia/Bishkek', b'Asia/Bishkek'), (b'Asia/Brunei', b'Asia/Brunei'), (b'Asia/Chita', b'Asia/Chita'), (b'Asia/Choibalsan', b'Asia/Choibalsan'), (b'Asia/Colombo', b'Asia/Colombo'), (b'Asia/Damascus', b'Asia/Damascus'), (b'Asia/Dhaka', b'Asia/Dhaka'), (b'Asia/Dili', b'Asia/Dili'), (b'Asia/Dubai', b'Asia/Dubai'), (b'Asia/Dushanbe', b'Asia/Dushanbe'), (b'Asia/Famagusta', b'Asia/Famagusta'), (b'Asia/Gaza', b'Asia/Gaza'), (b'Asia/Hebron', b'Asia/Hebron'), (b'Asia/Ho_Chi_Minh', b'Asia/Ho_Chi_Minh'), (b'Asia/Hong_Kong', b'Asia/Hong_Kong'), (b'Asia/Hovd', b'Asia/Hovd'), (b'Asia/Irkutsk', b'Asia/Irkutsk'), (b'Asia/Jakarta', b'Asia/Jakarta'), (b'Asia/Jayapura', b'Asia/Jayapura'), (b'Asia/Jerusalem', b'Asia/Jerusalem'), (b'Asia/Kabul', b'Asia/Kabul'), (b'Asia/Kamchatka', b'Asia/Kamchatka'), (b'Asia/Karachi', b'Asia/Karachi'), (b'Asia/Kathmandu', b'Asia/Kathmandu'), (b'Asia/Khandyga', b'Asia/Khandyga'), (b'Asia/Kolkata', b'Asia/Kolkata'), (b'Asia/Krasnoyarsk', b'Asia/Krasnoyarsk'), (b'Asia/Kuala_Lumpur', b'Asia/Kuala_Lumpur'), (b'Asia/Kuching', b'Asia/Kuching'), (b'Asia/Kuwait', b'Asia/Kuwait'), (b'Asia/Macau', b'Asia/Macau'), (b'Asia/Magadan', b'Asia/Magadan'), (b'Asia/Makassar', b'Asia/Makassar'), (b'Asia/Manila', b'Asia/Manila'), (b'Asia/Muscat', b'Asia/Muscat'), (b'Asia/Nicosia', b'Asia/Nicosia'), (b'Asia/Novokuznetsk', b'Asia/Novokuznetsk'), (b'Asia/Novosibirsk', b'Asia/Novosibirsk'), (b'Asia/Omsk', b'Asia/Omsk'), (b'Asia/Oral', b'Asia/Oral'), (b'Asia/Phnom_Penh', b'Asia/Phnom_Penh'), (b'Asia/Pontianak', b'Asia/Pontianak'), (b'Asia/Pyongyang', b'Asia/Pyongyang'), (b'Asia/Qatar', b'Asia/Qatar'), (b'Asia/Qostanay', b'Asia/Qostanay'), (b'Asia/Qyzylorda', b'Asia/Qyzylorda'), (b'Asia/Riyadh', b'Asia/Riyadh'), (b'Asia/Sakhalin', b'Asia/Sakhalin'), (b'Asia/Samarkand', b'Asia/Samarkand'), (b'Asia/Seoul', b'Asia/Seoul'), (b'Asia/Shanghai', b'Asia/Shanghai'), (b'Asia/Singapore', b'Asia/Singapore'), (b'Asia/Srednekolymsk', b'Asia/Srednekolymsk'), (b'Asia/Taipei', b'Asia/Taipei'), (b'Asia/Tashkent', b'Asia/Tashkent'), (b'Asia/Tbilisi', b'Asia/Tbilisi'), (b'Asia/Tehran', b'Asia/Tehran'), (b'Asia/Thimphu', b'Asia/Thimphu'), (b'Asia/Tokyo', b'Asia/Tokyo'), (b'Asia/Tomsk', b'Asia/Tomsk'), (b'Asia/Ulaanbaatar', b'Asia/Ulaanbaatar'), (b'Asia/Urumqi', b'Asia/Urumqi'), (b'Asia/Ust-Nera', b'Asia/Ust-Nera'), (b'Asia/Vientiane', b'Asia/Vientiane'), (b'Asia/Vladivostok', b'Asia/Vladivostok'), (b'Asia/Yakutsk', b'Asia/Yakutsk'), (b'Asia/Yangon', b'Asia/Yangon'), (b'Asia/Yekaterinburg', b'Asia/Yekaterinburg'), (b'Asia/Yerevan', b'Asia/Yerevan'), (b'Atlantic/Azores', b'Atlantic/Azores'), (b'Atlantic/Bermuda', b'Atlantic/Bermuda'), (b'Atlantic/Canary', b'Atlantic/Canary'), (b'Atlantic/Cape_Verde', b'Atlantic/Cape_Verde'), (b'Atlantic/Faroe', b'Atlantic/Faroe'), (b'Atlantic/Madeira', b'Atlantic/Madeira'), (b'Atlantic/Reykjavik', b'Atlantic/Reykjavik'), (b'Atlantic/South_Georgia', b'Atlantic/South_Georgia'), (b'Atlantic/St_Helena', b'Atlantic/St_Helena'), (b'Atlantic/Stanley', b'Atlantic/Stanley'), (b'Australia/Adelaide', b'Australia/Adelaide'), (b'Australia/Brisbane', b'Australia/Brisbane'), (b'Australia/Broken_Hill', b'Australia/Broken_Hill'), (b'Australia/Currie', b'Australia/Currie'), (b'Australia/Darwin', b'Australia/Darwin'), (b'Australia/Eucla', b'Australia/Eucla'), (b'Australia/Hobart', b'Australia/Hobart'), (b'Australia/Lindeman', b'Australia/Lindeman'), (b'Australia/Lord_Howe', b'Australia/Lord_Howe'), (b'Australia/Melbourne', b'Australia/Melbourne'), (b'Australia/Perth', b'Australia/Perth'), (b'Australia/Sydney', b'Australia/Sydney'), (b'Canada/Atlantic', b'Canada/Atlantic'), (b'Canada/Central', b'Canada/Central'), (b'Canada/Eastern', b'Canada/Eastern'), (b'Canada/Mountain', b'Canada/Mountain'), (b'Canada/Newfoundland', b'Canada/Newfoundland'), (b'Canada/Pacific', b'Canada/Pacific'), (b'Europe/Amsterdam', b'Europe/Amsterdam'), (b'Europe/Andorra', b'Europe/Andorra'), (b'Europe/Astrakhan', b'Europe/Astrakhan'), (b'Europe/Athens', b'Europe/Athens'), (b'Europe/Belgrade', b'Europe/Belgrade'), (b'Europe/Berlin', b'Europe/Berlin'), (b'Europe/Bratislava', b'Europe/Bratislava'), (b'Europe/Brussels', b'Europe/Brussels'), (b'Europe/Bucharest', b'Europe/Bucharest'), (b'Europe/Budapest', b'Europe/Budapest'), (b'Europe/Busingen', b'Europe/Busingen'), (b'Europe/Chisinau', b'Europe/Chisinau'), (b'Europe/Copenhagen', b'Europe/Copenhagen'), (b'Europe/Dublin', b'Europe/Dublin'), (b'Europe/Gibraltar', b'Europe/Gibraltar'), (b'Europe/Guernsey', b'Europe/Guernsey'), (b'Europe/Helsinki', b'Europe/Helsinki'), (b'Europe/Isle_of_Man', b'Europe/Isle_of_Man'), (b'Europe/Istanbul', b'Europe/Istanbul'), (b'Europe/Jersey', b'Europe/Jersey'), (b'Europe/Kaliningrad', b'Europe/Kaliningrad'), (b'Europe/Kiev', b'Europe/Kiev'), (b'Europe/Kirov', b'Europe/Kirov'), (b'Europe/Lisbon', b'Europe/Lisbon'), (b'Europe/Ljubljana', b'Europe/Ljubljana'), (b'Europe/London', b'Europe/London'), (b'Europe/Luxembourg', b'Europe/Luxembourg'), (b'Europe/Madrid', b'Europe/Madrid'), (b'Europe/Malta', b'Europe/Malta'), (b'Europe/Mariehamn', b'Europe/Mariehamn'), (b'Europe/Minsk', b'Europe/Minsk'), (b'Europe/Monaco', b'Europe/Monaco'), (b'Europe/Moscow', b'Europe/Moscow'), (b'Europe/Oslo', b'Europe/Oslo'), (b'Europe/Paris', b'Europe/Paris'), (b'Europe/Podgorica', b'Europe/Podgorica'), (b'Europe/Prague', b'Europe/Prague'), (b'Europe/Riga', b'Europe/Riga'), (b'Europe/Rome', b'Europe/Rome'), (b'Europe/Samara', b'Europe/Samara'), (b'Europe/San_Marino', b'Europe/San_Marino'), (b'Europe/Sarajevo', b'Europe/Sarajevo'), (b'Europe/Saratov', b'Europe/Saratov'), (b'Europe/Simferopol', b'Europe/Simferopol'), (b'Europe/Skopje', b'Europe/Skopje'), (b'Europe/Sofia', b'Europe/Sofia'), (b'Europe/Stockholm', b'Europe/Stockholm'), (b'Europe/Tallinn', b'Europe/Tallinn'), (b'Europe/Tirane', b'Europe/Tirane'), (b'Europe/Ulyanovsk', b'Europe/Ulyanovsk'), (b'Europe/Uzhgorod', b'Europe/Uzhgorod'), (b'Europe/Vaduz', b'Europe/Vaduz'), (b'Europe/Vatican', b'Europe/Vatican'), (b'Europe/Vienna', b'Europe/Vienna'), (b'Europe/Vilnius', b'Europe/Vilnius'), (b'Europe/Volgograd', b'Europe/Volgograd'), (b'Europe/Warsaw', b'Europe/Warsaw'), (b'Europe/Zagreb', b'Europe/Zagreb'), (b'Europe/Zaporozhye', b'Europe/Zaporozhye'), (b'Europe/Zurich', b'Europe/Zurich'), (b'GMT', b'GMT'), (b'Indian/Antananarivo', b'Indian/Antananarivo'), (b'Indian/Chagos', b'Indian/Chagos'), (b'Indian/Christmas', b'Indian/Christmas'), (b'Indian/Cocos', b'Indian/Cocos'), (b'Indian/Comoro', b'Indian/Comoro'), (b'Indian/Kerguelen', b'Indian/Kerguelen'), (b'Indian/Mahe', b'Indian/Mahe'), (b'Indian/Maldives', b'Indian/Maldives'), (b'Indian/Mauritius', b'Indian/Mauritius'), (b'Indian/Mayotte', b'Indian/Mayotte'), (b'Indian/Reunion', b'Indian/Reunion'), (b'Pacific/Apia', b'Pacific/Apia'), (b'Pacific/Auckland', b'Pacific/Auckland'), (b'Pacific/Bougainville', b'Pacific/Bougainville'), (b'Pacific/Chatham', b'Pacific/Chatham'), (b'Pacific/Chuuk', b'Pacific/Chuuk'), (b'Pacific/Easter', b'Pacific/Easter'), (b'Pacific/Efate', b'Pacific/Efate'), (b'Pacific/Enderbury', b'Pacific/Enderbury'), (b'Pacific/Fakaofo', b'Pacific/Fakaofo'), (b'Pacific/Fiji', b'Pacific/Fiji'), (b'Pacific/Funafuti', b'Pacific/Funafuti'), (b'Pacific/Galapagos', b'Pacific/Galapagos'), (b'Pacific/Gambier', b'Pacific/Gambier'), (b'Pacific/Guadalcanal', b'Pacific/Guadalcanal'), (b'Pacific/Guam', b'Pacific/Guam'), (b'Pacific/Honolulu', b'Pacific/Honolulu'), (b'Pacific/Kiritimati', b'Pacific/Kiritimati'), (b'Pacific/Kosrae', b'Pacific/Kosrae'), (b'Pacific/Kwajalein', b'Pacific/Kwajalein'), (b'Pacific/Majuro', b'Pacific/Majuro'), (b'Pacific/Marquesas', b'Pacific/Marquesas'), (b'Pacific/Midway', b'Pacific/Midway'), (b'Pacific/Nauru', b'Pacific/Nauru'), (b'Pacific/Niue', b'Pacific/Niue'), (b'Pacific/Norfolk', b'Pacific/Norfolk'), (b'Pacific/Noumea', b'Pacific/Noumea'), (b'Pacific/Pago_Pago', b'Pacific/Pago_Pago'), (b'Pacific/Palau', b'Pacific/Palau'), (b'Pacific/Pitcairn', b'Pacific/Pitcairn'), (b'Pacific/Pohnpei', b'Pacific/Pohnpei'), (b'Pacific/Port_Moresby', b'Pacific/Port_Moresby'), (b'Pacific/Rarotonga', b'Pacific/Rarotonga'), (b'Pacific/Saipan', b'Pacific/Saipan'), (b'Pacific/Tahiti', b'Pacific/Tahiti'), (b'Pacific/Tarawa', b'Pacific/Tarawa'), (b'Pacific/Tongatapu', b'Pacific/Tongatapu'), (b'Pacific/Wake', b'Pacific/Wake'), (b'Pacific/Wallis', b'Pacific/Wallis'), (b'US/Alaska', b'US/Alaska'), (b'US/Arizona', b'US/Arizona'), (b'US/Central', b'US/Central'), (b'US/Eastern', b'US/Eastern'), (b'US/Hawaii', b'US/Hawaii'), (b'US/Mountain', b'US/Mountain'), (b'US/Pacific', b'US/Pacific'), (b'UTC', b'UTC')], max_length=48, verbose_name='Timezone'),
        ),
    ]



from __future__ import unicode_literals

from django.apps import apps
from django.db.models.signals import post_migrate, post_save
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.acls.links import link_acl_list
from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.classes import ModelProperty
from mayan.apps.common.links import link_object_error_list
from mayan.apps.common.html_widgets import TwoStateWidget
from mayan.apps.common.menus import (
    menu_facet, menu_list_facet, menu_main, menu_object, menu_secondary,
    menu_setup, menu_tools
)
from mayan.apps.common.permissions_runtime import permission_error_log_view
from mayan.apps.events.classes import ModelEventType
from mayan.apps.events.links import (
    link_events_for_object, link_object_event_types_user_subcriptions_list
)
from mayan.apps.navigation.classes import SourceColumn

from .classes import DocumentStateHelper, WorkflowAction
from .events import event_workflow_edited
from .handlers import (
    handler_create_workflow_image_cache, handler_index_document,
    handler_launch_workflow, handler_trigger_transition
)
from .html_widgets import WorkflowLogExtraDataWidget, widget_transition_events
from .links import (
    link_workflow_instance_list, link_document_type_workflow_templates,
    link_workflow_template_document_types, link_workflow_template_create,
    link_workflow_template_delete, link_workflow_template_edit,
    link_workflow_template_launch, link_workflow_template_list,
    link_workflow_template_state_list,
    link_workflow_template_state_action_delete,
    link_workflow_template_state_action_edit,
    link_workflow_template_state_action_list,
    link_workflow_template_state_action_selection,
    link_workflow_template_state_create, link_workflow_template_state_delete,
    link_workflow_template_state_edit, link_workflow_template_transition_list,
    link_workflow_template_transition_create,
    link_workflow_template_transition_delete, link_workflow_template_transition_edit,
    link_workflow_template_transition_field_create,
    link_workflow_template_transition_field_delete,
    link_workflow_template_transition_field_edit,
    link_workflow_template_transition_field_list,
    link_tool_launch_workflows, link_workflow_instance_detail,
    link_workflow_instance_transition, link_workflow_runtime_proxy_document_list,
    link_workflow_runtime_proxy_list, link_workflow_template_preview,
    link_workflow_runtime_proxy_state_document_list, link_workflow_runtime_proxy_state_list,
    link_workflow_template_transition_events
)
from .permissions import (
    permission_workflow_delete, permission_workflow_edit,
    permission_workflow_tools, permission_workflow_transition,
    permission_workflow_view
)


class DocumentStatesApp(MayanAppConfig):
    app_namespace = 'document_states'
    app_url = 'workflows'
    has_rest_api = True
    has_tests = True
    name = 'mayan.apps.document_states'
    verbose_name = _('Workflows')

    def ready(self):
        super(DocumentStatesApp, self).ready()
        from actstream import registry

        Action = apps.get_model(
            app_label='actstream', model_name='Action'
        )
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )
        DocumentType = apps.get_model(
            app_label='documents', model_name='DocumentType'
        )
        ErrorLogEntry = apps.get_model(
            app_label='common', model_name='ErrorLogEntry'
        )

        Workflow = self.get_model('Workflow')
        WorkflowInstance = self.get_model('WorkflowInstance')
        WorkflowInstanceLogEntry = self.get_model('WorkflowInstanceLogEntry')
        WorkflowRuntimeProxy = self.get_model('WorkflowRuntimeProxy')
        WorkflowState = self.get_model('WorkflowState')
        WorkflowStateAction = self.get_model('WorkflowStateAction')
        WorkflowStateRuntimeProxy = self.get_model('WorkflowStateRuntimeProxy')
        WorkflowTransition = self.get_model('WorkflowTransition')
        WorkflowTransitionField = self.get_model('WorkflowTransitionField')
        WorkflowTransitionTriggerEvent = self.get_model(
            'WorkflowTransitionTriggerEvent'
        )

        Document.add_to_class(
            name='workflow', value=DocumentStateHelper.constructor
        )

        ErrorLogEntry.objects.register(model=WorkflowStateAction)

        WorkflowAction.initialize()

        ModelProperty(
            model=Document,
            name='workflow.< workflow internal name >.get_current_state',
            label=_('Current state of a workflow'), description=_(
                'Return the current state of the selected workflow.'
            )
        )
        ModelProperty(
            model=Document,
            name='workflow.< workflow internal name >.get_current_state.completion',
            label=_('Current state of a workflow'), description=_(
                'Return the completion value of the current state of the '
                'selected workflow.'
            )
        )

        ModelEventType.register(
            event_types=(event_workflow_edited,), model=Workflow
        )

        ModelPermission.register(
            model=Document, permissions=(permission_workflow_view,)
        )
        ModelPermission.register(
            model=Workflow, permissions=(
                permission_error_log_view, permission_workflow_delete,
                permission_workflow_edit, permission_workflow_tools,
                permission_workflow_transition, permission_workflow_view
            )
        )
        ModelPermission.register(
            model=WorkflowTransition,
            permissions=(permission_workflow_transition,)
        )

        ModelPermission.register_inheritance(
            model=WorkflowInstance, related='workflow',
        )
        ModelPermission.register_inheritance(
            model=WorkflowInstanceLogEntry,
            related='workflow_instance__workflow',
        )
        ModelPermission.register_inheritance(
            model=WorkflowState, related='workflow',
        )
        ModelPermission.register_inheritance(
            model=WorkflowStateAction, related='state__workflow',
        )
        ModelPermission.register_inheritance(
            model=WorkflowTransition, related='workflow',
        )
        ModelPermission.register_inheritance(
            model=WorkflowTransitionField, related='transition',
        )
        ModelPermission.register_inheritance(
            model=WorkflowTransitionTriggerEvent,
            related='transition__workflow',
        )

        SourceColumn(
            attribute='label', is_sortable=True, source=Workflow
        )
        column_workflow_internal_name = SourceColumn(
            attribute='internal_name', is_sortable=True, source=Workflow
        )
        column_workflow_internal_name.add_exclude(source=WorkflowRuntimeProxy)
        SourceColumn(
            attribute='get_initial_state', empty_value=_('None'),
            source=Workflow
        )

        SourceColumn(
            source=WorkflowInstance, label=_('Current state'),
            attribute='get_current_state'
        )
        SourceColumn(
            source=WorkflowInstance, label=_('User'),
            func=lambda context: getattr(
                context['object'].get_last_log_entry(), 'user', _('None')
            )
        )
        SourceColumn(
            source=WorkflowInstance, label=_('Last transition'),
            attribute='get_last_transition'
        )
        SourceColumn(
            source=WorkflowInstance, label=_('Date and time'),
            func=lambda context: getattr(
                context['object'].get_last_log_entry(), 'datetime', _('None')
            )
        )
        SourceColumn(
            source=WorkflowInstance, label=_('Completion'),
            func=lambda context: getattr(
                context['object'].get_current_state(), 'completion', _('None')
            )
        )

        SourceColumn(
            source=WorkflowInstanceLogEntry, label=_('Date and time'),
            attribute='datetime'
        )
        SourceColumn(
            source=WorkflowInstanceLogEntry, label=_('User'), attribute='user'
        )
        SourceColumn(
            source=WorkflowInstanceLogEntry,
            attribute='transition__origin_state', is_sortable=True
        )
        SourceColumn(
            source=WorkflowInstanceLogEntry,
            attribute='transition', is_sortable=True
        )
        SourceColumn(
            source=WorkflowInstanceLogEntry,
            attribute='transition__destination_state', is_sortable=True
        )
        SourceColumn(
            source=WorkflowInstanceLogEntry,
            attribute='comment', is_sortable=True
        )
        SourceColumn(
            source=WorkflowInstanceLogEntry,
            attribute='get_extra_data', label=_('Additional details'),
            widget=WorkflowLogExtraDataWidget
        )

        SourceColumn(
            attribute='label', is_sortable=True, source=WorkflowState
        )
        SourceColumn(
            attribute='initial', is_sortable=True, source=WorkflowState,
            widget=TwoStateWidget
        )
        SourceColumn(
            attribute='completion', source=WorkflowState, is_sortable=True,
        )

        SourceColumn(
            attribute='label', is_sortable=True, source=WorkflowStateAction
        )
        SourceColumn(
            attribute='enabled', is_sortable=True, source=WorkflowStateAction,
            widget=TwoStateWidget
        )
        SourceColumn(
            attribute='get_when_display', label=_('When?'),
            source=WorkflowStateAction
        )
        SourceColumn(
            attribute='get_class_label', label=_('Action type'),
            source=WorkflowStateAction
        )

        SourceColumn(
            attribute='label', is_sortable=True, source=WorkflowTransition,
        )
        SourceColumn(
            attribute='origin_state', is_sortable=True,
            source=WorkflowTransition
        )
        SourceColumn(
            attribute='destination_state', is_sortable=True,
            source=WorkflowTransition
        )
        SourceColumn(
            source=WorkflowTransition, label=_('Triggers'),
            func=lambda context: widget_transition_events(
                transition=context['object']
            )
        )

        SourceColumn(
            attribute='name', is_identifier=True, is_sortable=True,
            source=WorkflowTransitionField
        )
        SourceColumn(
            attribute='label', is_sortable=True, source=WorkflowTransitionField
        )
        SourceColumn(
            attribute='get_field_type_display', label=_('Type'),
            source=WorkflowTransitionField
        )
        SourceColumn(
            attribute='required', is_sortable=True,
            source=WorkflowTransitionField, widget=TwoStateWidget
        )
        SourceColumn(
            attribute='get_widget_display', label=_('Widget'),
            is_sortable=False, source=WorkflowTransitionField
        )
        SourceColumn(
            attribute='widget_kwargs', is_sortable=True,
            source=WorkflowTransitionField
        )

        SourceColumn(
            source=WorkflowRuntimeProxy, label=_('Documents'),
            func=lambda context: context['object'].get_document_count(
                user=context['request'].user
            ), order=99
        )
        SourceColumn(
            source=WorkflowStateRuntimeProxy, label=_('Documents'),
            func=lambda context: context['object'].get_document_count(
                user=context['request'].user
            ), order=99
        )

        menu_facet.bind_links(
            links=(link_workflow_instance_list,), sources=(Document,)
        )

        menu_list_facet.bind_links(
            links=(
                link_acl_list, link_events_for_object,
                link_object_event_types_user_subcriptions_list,
                link_workflow_template_document_types,
                link_workflow_template_state_list, link_workflow_template_transition_list,
                link_workflow_template_preview
            ), sources=(Workflow,)
        )

        menu_list_facet.unbind_links(
            links=(
                link_acl_list, link_events_for_object,
                link_object_event_types_user_subcriptions_list,
                link_workflow_template_document_types,
                link_workflow_template_state_list, link_workflow_template_transition_list,
                link_workflow_template_preview
            ), sources=(WorkflowRuntimeProxy,)
        )

        menu_list_facet.bind_links(
            links=(
                link_document_type_workflow_templates,
            ), sources=(DocumentType,)
        )

        menu_main.bind_links(links=(link_workflow_runtime_proxy_list,), position=10)
        menu_object.bind_links(
            links=(
                link_workflow_template_delete, link_workflow_template_edit,
                link_workflow_template_launch
            ), sources=(Workflow,)
        )
        menu_object.bind_links(
            links=(
                link_workflow_template_state_edit,
                link_workflow_template_state_action_list,
                link_workflow_template_state_delete
            ), sources=(WorkflowState,)
        )
        menu_object.bind_links(
            links=(
                link_workflow_template_transition_edit,
                link_workflow_template_transition_events,
                link_workflow_template_transition_field_list, link_acl_list,
                link_workflow_template_transition_delete
            ), sources=(WorkflowTransition,)
        )
        menu_object.bind_links(
            links=(
                link_workflow_template_transition_field_delete,
                link_workflow_template_transition_field_edit
            ), sources=(WorkflowTransitionField,)
        )
        menu_object.bind_links(
            links=(
                link_workflow_instance_detail,
                link_workflow_instance_transition
            ), sources=(WorkflowInstance,)
        )

        menu_list_facet.bind_links(
            links=(
                link_workflow_runtime_proxy_document_list,
                link_workflow_runtime_proxy_state_list,
            ), sources=(WorkflowRuntimeProxy,)
        )
        menu_list_facet.bind_links(
            links=(
                link_workflow_runtime_proxy_state_document_list,
            ), sources=(WorkflowStateRuntimeProxy,)
        )
        menu_object.bind_links(
            links=(
                link_workflow_template_state_action_edit,
                link_object_error_list,
                link_workflow_template_state_action_delete,
            ), sources=(WorkflowStateAction,)
        )

        menu_secondary.bind_links(
            links=(link_workflow_template_list, link_workflow_template_create),
            sources=(
                Workflow, 'document_states:workflow_template_create',
                'document_states:workflow_template_list'
            )
        )
        menu_secondary.bind_links(
            links=(link_workflow_template_transition_field_create,),
            sources=(
                WorkflowTransition,
            )
        )
        menu_secondary.bind_links(
            links=(link_workflow_runtime_proxy_list,),
            sources=(
                WorkflowRuntimeProxy,
            )
        )
        menu_secondary.bind_links(
            links=(link_workflow_template_state_action_selection,),
            sources=(
                WorkflowState,
            )
        )
        menu_secondary.bind_links(
            links=(
                link_workflow_template_transition_create,
            ), sources=(
                WorkflowTransition,
                'document_states:workflow_template_transition_create',
                'document_states:workflow_template_transition_list',
            )
        )
        menu_secondary.bind_links(
            links=(
                link_workflow_template_state_create,
            ), sources=(
                WorkflowState,
                'document_states:workflow_template_state_create',
                'document_states:workflow_template_state_list',
            )
        )

        menu_setup.bind_links(links=(link_workflow_template_list,))

        menu_tools.bind_links(links=(link_tool_launch_workflows,))

        post_save.connect(
            dispatch_uid='workflows_handler_launch_workflow',
            receiver=handler_launch_workflow,
            sender=Document
        )

        # Index updating

        post_migrate.connect(
            dispatch_uid='workflows_handler_create_workflow_image_cache',
            receiver=handler_create_workflow_image_cache,
        )
        post_save.connect(
            dispatch_uid='workflows_handler_index_document_save',
            receiver=handler_index_document,
            sender=WorkflowInstanceLogEntry
        )
        post_save.connect(
            dispatch_uid='workflows_handler_trigger_transition',
            receiver=handler_trigger_transition,
            sender=Action
        )

        registry.register(Workflow)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.acls.links import link_acl_list
from mayan.apps.acls.permissions import permission_acl_edit, permission_acl_view
from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import (
    menu_list_facet, menu_multi_item, menu_object, menu_secondary, menu_tools
)
from mayan.apps.events.classes import ModelEventType
from mayan.apps.events.links import (
    link_events_for_object, link_object_event_types_user_subcriptions_list
)
from mayan.apps.navigation.classes import SourceColumn

from .events import event_cache_edited, event_cache_purged
from .links import (
    link_caches_list, link_cache_multiple_purge, link_cache_purge
)
from .permissions import permission_cache_purge, permission_cache_view


class FileCachingConfig(MayanAppConfig):
    app_namespace = 'file_caching'
    app_url = 'file_caching'
    has_tests = True
    name = 'mayan.apps.file_caching'
    verbose_name = _('File caching')

    def ready(self):
        super(FileCachingConfig, self).ready()
        from actstream import registry

        Cache = self.get_model(model_name='Cache')

        ModelEventType.register(
            event_types=(event_cache_edited, event_cache_purged,),
            model=Cache
        )

        ModelPermission.register(
            model=Cache, permissions=(
                permission_acl_edit, permission_acl_view,
                permission_cache_purge, permission_cache_view
            )
        )

        SourceColumn(attribute='label', is_sortable=True, source=Cache)
        SourceColumn(attribute='name', is_sortable=True, source=Cache)
        SourceColumn(
            attribute='storage_instance_path', is_sortable=True, source=Cache
        )
        SourceColumn(
            attribute='get_maximum_size_display', is_sortable=True,
            sort_field='maximum_size', source=Cache
        )
        SourceColumn(attribute='get_total_size_display', source=Cache)

        menu_list_facet.bind_links(
            links=(
                link_acl_list, link_events_for_object,
                link_object_event_types_user_subcriptions_list,
            ), sources=(Cache,)
        )

        menu_object.bind_links(
            links=(link_cache_purge,),
            sources=(Cache,)
        )
        menu_multi_item.bind_links(
            links=(link_cache_multiple_purge,),
            sources=(Cache,)
        )
        menu_secondary.bind_links(
            links=(link_caches_list,), sources=(
                Cache,
            )
        )

        menu_tools.bind_links(links=(link_caches_list,))

        registry.register(Cache)



from __future__ import absolute_import, unicode_literals

TEST_CABINET_LABEL = 'test cabinet label'
TEST_CABINET_LABEL_EDITED = 'test cabinet edited label'
TEST_CABINET_CHILD_LABEL = 'test cabinet child label'



from __future__ import unicode_literals

from django.conf.urls import url

from .views import NamespaceDetailView, NamespaceListView, SettingEditView

urlpatterns = [
    url(
        regex=r'^namespaces/$', view=NamespaceListView.as_view(),
        name='namespace_list'
    ),
    url(
        regex=r'^namespaces/(?P<namespace_name>\w+)/$',
        view=NamespaceDetailView.as_view(), name='namespace_detail'
    ),
    url(
        regex=r'^namespaces/settings/(?P<setting_global_name>\w+)/edit/$',
        view=SettingEditView.as_view(), name='setting_edit_view'
    ),
]



from __future__ import unicode_literals

from django.conf.urls import url

from .api_views import (
    APIDocumentOCRView, APIDocumentPageOCRContentView,
    APIDocumentVersionOCRView
)
from .views import (
    DocumentOCRContentDeleteView, DocumentOCRContentView,
    DocumentOCRDownloadView,
    DocumentOCRErrorsListView, DocumentPageOCRContentView, DocumentSubmitView,
    DocumentTypeSettingsEditView, DocumentTypeSubmitView, EntryListView
)

urlpatterns = [
    url(
        regex=r'^documents/(?P<pk>\d+)/content/$',
        view=DocumentOCRContentView.as_view(), name='document_ocr_content'
    ),
    url(
        regex=r'^documents/(?P<pk>\d+)/content/delete/$',
        view=DocumentOCRContentDeleteView.as_view(),
        name='document_ocr_content_delete'
    ),
    url(
        regex=r'^documents/multiple/content/delete/$',
        view=DocumentOCRContentDeleteView.as_view(),
        name='document_ocr_content_delete_multiple'
    ),
    url(
        regex=r'^documents/(?P<pk>\d+)/ocr/download/$',
        view=DocumentOCRDownloadView.as_view(), name='document_ocr_download'
    ),
    url(
        regex=r'^documents/(?P<pk>\d+)/ocr/errors/$',
        view=DocumentOCRErrorsListView.as_view(),
        name='document_ocr_error_list'
    ),
    url(
        regex=r'^documents/(?P<pk>\d+)/submit/$',
        view=DocumentSubmitView.as_view(), name='document_submit'
    ),
    url(
        regex=r'^documents/multiple/submit/$',
        view=DocumentSubmitView.as_view(), name='document_submit_multiple'
    ),
    url(
        regex=r'^documents/pages/(?P<pk>\d+)/content/$',
        view=DocumentPageOCRContentView.as_view(),
        name='document_page_ocr_content'
    ),
    url(
        regex=r'^document_types/submit/$',
        view=DocumentTypeSubmitView.as_view(), name='document_type_submit'
    ),
    url(
        regex=r'^document_types/(?P<pk>\d+)/ocr/settings/$',
        view=DocumentTypeSettingsEditView.as_view(),
        name='document_type_ocr_settings'
    ),
    url(regex=r'^logs/$', view=EntryListView.as_view(), name='entry_list'),
]

api_urls = [
    url(
        regex=r'^documents/(?P<pk>\d+)/ocr/submit/$',
        view=APIDocumentOCRView.as_view(), name='document-ocr-submit-view'
    ),
    url(
        regex=r'^documents/(?P<document_pk>\d+)/versions/(?P<version_pk>\d+)/ocr/$',
        view=APIDocumentVersionOCRView.as_view(),
        name='document-version-ocr-submit-view'
    ),
    url(
        regex=r'^documents/(?P<document_pk>\d+)/versions/(?P<version_pk>\d+)/pages/(?P<page_pk>\d+)/ocr/$',
        view=APIDocumentPageOCRContentView.as_view(),
        name='document-page-ocr-content-view'
    ),
]



from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('documents', '0041_auto_20170823_1855'),
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentPageContent',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'content', models.TextField(
                        blank=True, verbose_name='Content'
                    )
                ),
                (
                    'document_page', models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='content', to='documents.DocumentPage',
                        verbose_name='Document page'
                    )
                ),
            ],
            options={
                'verbose_name': 'Document page content',
                'verbose_name_plural': 'Document pages contents',
            },
        ),
        migrations.CreateModel(
            name='DocumentVersionParseError',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'datetime_submitted', models.DateTimeField(
                        auto_now_add=True, db_index=True,
                        verbose_name='Date time submitted'
                    )
                ),
                (
                    'result', models.TextField(
                        blank=True, null=True, verbose_name='Result'
                    )
                ),
                (
                    'document_version', models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='parse_errors',
                        to='documents.DocumentVersion',
                        verbose_name='Document version'
                    )
                ),
            ],
            options={
                'ordering': ('datetime_submitted',),
                'verbose_name': 'Document version parse error',
                'verbose_name_plural': 'Document version parse errors',
            },
        ),
    ]



from __future__ import absolute_import, unicode_literals

from datetime import date
import os
import shutil

import gnupg

from mayan.apps.storage.utils import mkdtemp


class GPGBackend(object):
    def __init__(self, **kwargs):
        self.kwargs = kwargs


class PythonGNUPGBackend(GPGBackend):
    @staticmethod
    def _import_key(gpg, **kwargs):
        return gpg.import_keys(**kwargs)

    @staticmethod
    def _list_keys(gpg, **kwargs):
        return gpg.list_keys(**kwargs)

    @staticmethod
    def _import_and_list_keys(gpg, **kwargs):
        import_results = gpg.import_keys(**kwargs)
        return import_results, gpg.list_keys(
            keys=import_results.fingerprints[0]
        )[0]

    @staticmethod
    def _sign_file(gpg, file_object, key_data, passphrase, clearsign, detached, binary, output):
        import_results = gpg.import_keys(key_data=key_data)

        return gpg.sign_file(
            file=file_object, keyid=import_results.fingerprints[0],
            passphrase=passphrase, clearsign=clearsign, detach=detached,
            binary=binary, output=output
        )

    @staticmethod
    def _decrypt_file(gpg, file_object, keys):
        for key in keys:
            gpg.import_keys(key_data=key['key_data'])

        return gpg.decrypt_file(file=file_object)

    @staticmethod
    def _verify_file(gpg, file_object, keys, data_filename=None):
        for key in keys:
            gpg.import_keys(key_data=key['key_data'])

        return gpg.verify_file(
            file=file_object, data_filename=data_filename
        )

    @staticmethod
    def _recv_keys(gpg, keyserver, key_id):
        import_results = gpg.recv_keys(keyserver, key_id)
        if import_results.count:
            key_data = gpg.export_keys(import_results.fingerprints[0])
        else:
            key_data = None
        return key_data

    @staticmethod
    def _search_keys(gpg, keyserver, query):
        return gpg.search_keys(
            keyserver=keyserver, query=query
        )

    def gpg_command(self, function, **kwargs):
        temporary_directory = mkdtemp()
        os.chmod(temporary_directory, 0x1C0)

        gpg = gnupg.GPG(
            gnupghome=temporary_directory, gpgbinary=self.kwargs['binary_path']
        )

        result = function(gpg=gpg, **kwargs)

        shutil.rmtree(temporary_directory)

        return result

    def import_key(self, key_data):
        return self.gpg_command(
            function=PythonGNUPGBackend._import_key, key_data=key_data
        )

    def list_keys(self, keys):
        return self.gpg_command(
            function=PythonGNUPGBackend._list_keys, keys=keys
        )

    def import_and_list_keys(self, key_data):
        return self.gpg_command(
            function=PythonGNUPGBackend._import_and_list_keys,
            key_data=key_data
        )

    def sign_file(self, file_object, key_data, passphrase, clearsign, detached, binary, output):
        return self.gpg_command(
            function=PythonGNUPGBackend._sign_file, file_object=file_object,
            key_data=key_data, passphrase=passphrase, clearsign=clearsign,
            detached=detached, binary=binary, output=output
        )

    def decrypt_file(self, file_object, keys):
        return self.gpg_command(
            function=PythonGNUPGBackend._decrypt_file, file_object=file_object,
            keys=keys
        )

    def verify_file(self, file_object, keys, data_filename=None):
        return self.gpg_command(
            function=PythonGNUPGBackend._verify_file, file_object=file_object,
            keys=keys, data_filename=data_filename
        )

    def recv_keys(self, keyserver, key_id):
        return self.gpg_command(
            function=PythonGNUPGBackend._recv_keys, keyserver=keyserver,
            key_id=key_id
        )

    def search_keys(self, keyserver, query):
        return self.gpg_command(
            function=PythonGNUPGBackend._search_keys, keyserver=keyserver,
            query=query
        )


class KeyStub(object):
    def __init__(self, raw):
        self.fingerprint = raw['keyid']
        self.key_type = raw['type']
        self.date = date.fromtimestamp(int(raw['date']))
        if raw['expires']:
            self.expires = date.fromtimestamp(int(raw['expires']))
        else:
            self.expires = None
        self.length = raw['length']
        self.user_id = raw['uids']

    @property
    def key_id(self):
        return self.fingerprint[-8:]


class SignatureVerification(object):
    def __init__(self, raw):
        self.user_id = raw['username']
        self.status = raw['status']
        self.key_id = raw['key_id']
        self.pubkey_fingerprint = raw['pubkey_fingerprint']
        self.date = date.fromtimestamp(int(raw['timestamp']))
        if raw['expire_timestamp']:
            self.expires = date.fromtimestamp(int(raw['expire_timestamp']))
        else:
            self.expires = None
        self.trust_text = raw['trust_text']
        self.valid = raw['valid']
        self.stderr = raw['stderr']
        self.fingerprint = raw['fingerprint']
        self.signature_id = raw['signature_id']
        self.trust_level = raw['trust_level']



from __future__ import unicode_literals

from ..classes import EventTypeNamespace

from .literals import (
    TEST_EVENT_TYPE_LABEL, TEST_EVENT_TYPE_NAME,
    TEST_EVENT_TYPE_NAMESPACE_LABEL, TEST_EVENT_TYPE_NAMESPACE_NAME
)


class EventTypeTestMixin(object):
    def _create_test_event_type(self):
        self.test_event_type_namespace = EventTypeNamespace(
            label=TEST_EVENT_TYPE_NAMESPACE_LABEL,
            name=TEST_EVENT_TYPE_NAMESPACE_NAME
        )
        self.test_event_type = self.test_event_type_namespace.add_event_type(
            label=TEST_EVENT_TYPE_LABEL,
            name=TEST_EVENT_TYPE_NAME
        )



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .permissions import permission_dependencies_view

link_check_version = Link(
    icon_class_path='mayan.apps.dependencies.icons.icon_check_version',
    text=_('Check for updates'), view='dependencies:check_version_view'
)

link_dependency_group_list = Link(
    icon_class_path='mayan.apps.dependencies.icons.icon_dependency_group_list',
    permissions=(permission_dependencies_view,),
    text=_('Groups'),
    view='dependencies:dependency_group_list',
)

link_dependency_group_entry_list = Link(
    args='resolved_object.name',
    icon_class_path='mayan.apps.dependencies.icons.icon_dependency_group_entry_list',
    permissions=(permission_dependencies_view,),
    text=_('Entries'),
    view='dependencies:dependency_group_entry_list',
)

link_dependency_group_entry_detail = Link(
    args=('resolved_object.dependency_group.name', 'resolved_object.name'),
    icon_class_path='mayan.apps.dependencies.icons.icon_dependency_group_entry_detail',
    permissions=(permission_dependencies_view,),
    text=_('Details'),
    view='dependencies:dependency_group_entry_detail',
)
link_packages_licenses = Link(
    icon_class_path='mayan.apps.dependencies.icons.icon_dependency_licenses',
    text=_('Dependencies licenses'), view='dependencies:dependency_licenses_view'
)
link_dependency_tool = Link(
    icon_class_path='mayan.apps.dependencies.icons.icon_dependency_group_list',
    permissions=(permission_dependencies_view,),
    text=_('Dependencies'),
    view='dependencies:dependency_group_list',
)



from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0052_auto_20191130_2209'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='document_type',
            field=models.ForeignKey(
                help_text='The document type of the document.',
                on_delete=django.db.models.deletion.CASCADE,
                related_name='documents', to='documents.DocumentType',
                verbose_name='Document type'
            ),
        ),
    ]



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(label=_('File metadata'), name='file_metadata')

permission_document_type_file_metadata_setup = namespace.add_permission(
    label=_('Change document type file metadata settings'),
    name='file_metadata_document_type_setup'
)
permission_file_metadata_submit = namespace.add_permission(
    label=_(
        'Submit document for file metadata processing'
    ), name='file_metadata_submit'
)
permission_file_metadata_view = namespace.add_permission(
    label=_('View file metadata'), name='file_metadata_view'
)



from __future__ import unicode_literals

from django.http import Http404
from django.utils.encoding import force_text

from .classes import SearchModel


class SearchModelMixin(object):
    def get_search_model_name(self):
        return self.kwargs.get(
            'search_model', self.request.GET.get(
                '_search_model', self.request.POST.get('_search_model')
            )
        )

    def get_search_model(self):
        try:
            return SearchModel.get(self.get_search_model_name())
        except KeyError as exception:
            raise Http404(force_text(exception))



from __future__ import print_function, unicode_literals

from pathlib2 import Path
import shutil

from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.storage.utils import mkdtemp

from ..utils import patch_files


class PatchFilesTestCase(BaseTestCase):
    test_replace_text = 'replaced_text'

    def setUp(self):
        super(PatchFilesTestCase, self).setUp()

        self.temporary_directory = mkdtemp()
        self.path_temporary_directory = Path(self.temporary_directory)
        self.path_test_file = self.path_temporary_directory / 'test_file.txt'

        with self.path_test_file.open(mode='w') as file_object:
            file_object.writelines(
                [
                    'line 1\n',
                    '    line 2\n',
                    'line 3\n',
                ]
            )

    def tearDown(self):
        super(PatchFilesTestCase, self).tearDown()
        shutil.rmtree(self.temporary_directory, ignore_errors=True)

    def _patch_test_file(self):
        replace_list = [
            {
                'filename_pattern': '*',
                'content_patterns': [
                    {
                        'search': self.test_search_text,
                        'replace': self.test_replace_text,
                    }
                ]
            }
        ]
        patch_files(
            path=self.path_temporary_directory, replace_list=replace_list
        )

        with self.path_test_file.open(mode='r') as file_object:
            self.final_text = file_object.read()

    def test_file_patching_single_line(self):
        self.test_search_text = 'line 1'

        self._patch_test_file()

        self.assertEqual(self.final_text, 'replaced_text\n    line 2\nline 3\n')

    def test_file_patching_multi_line(self):
        self.test_search_text = 'line 2\nline 3\n'

        self._patch_test_file()

        self.assertEqual(self.final_text, 'line 1\n    replaced_text')

    def test_file_patching_spaces(self):
        self.test_search_text = '    line 2'

        self._patch_test_file()

        self.assertEqual(self.final_text, 'line 1\nreplaced_text\nline 3\n')

    def test_file_patching_no_matches(self):
        self.test_search_text = 'line 4'

        self._patch_test_file()

        self.assertEqual(self.final_text, 'line 1\n    line 2\nline 3\n')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2019-12-19 02:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sources', '0023_auto_20191213_0044'),
    ]

    operations = [
        migrations.AlterField(
            model_name='intervalbasemodel',
            name='document_type',
            field=models.ForeignKey(help_text='Assign a document type to documents uploaded from this source.', on_delete=django.db.models.deletion.CASCADE, related_name='interval_sources', to='documents.DocumentType', verbose_name='Document type'),
        ),
    ]



from __future__ import unicode_literals

from ..widgets import DocumentPageThumbnailWidget

from .test_models import GenericDocumentTestCase


class DocumentPageWidgetTestCase(GenericDocumentTestCase):
    def test_document_list_view_document_with_no_pages(self):
        document_thumbnail_widget = DocumentPageThumbnailWidget()
        self.test_document.pages.all().delete()
        result = document_thumbnail_widget.render(instance=self.test_document)

        self.assertTrue(self.test_document.get_absolute_url() in result)



from __future__ import unicode_literals

from django.conf.urls import url

from .api_views import (
    APIDocumentCabinetListView, APICabinetDocumentListView,
    APICabinetDocumentView, APICabinetListView, APICabinetView
)
from .views import (
    DocumentAddToCabinetView, DocumentCabinetListView,
    DocumentRemoveFromCabinetView, CabinetChildAddView, CabinetCreateView,
    CabinetDeleteView, CabinetDetailView, CabinetEditView, CabinetListView,
)

urlpatterns_cabinets = [
    url(
        regex=r'^cabinets/$', view=CabinetListView.as_view(), name='cabinet_list'
    ),
    url(
        regex=r'^cabinets/create/$', view=CabinetCreateView.as_view(),
        name='cabinet_create'
    ),
    url(
        regex=r'^cabinets/(?P<pk>\d+)/children/add/$', view=CabinetChildAddView.as_view(),
        name='cabinet_child_add'
    ),
    url(
        regex=r'^cabinets/(?P<pk>\d+)/delete/$', view=CabinetDeleteView.as_view(),
        name='cabinet_delete'
    ),
    url(
        regex=r'^cabinets/(?P<pk>\d+)/edit/$', view=CabinetEditView.as_view(),
        name='cabinet_edit'
    ),
    url(
        regex=r'^cabinets/(?P<pk>\d+)/$', view=CabinetDetailView.as_view(),
        name='cabinet_view'
    ),
]

urlpatterns_documents_cabinets = [
    url(
        regex=r'^documents/(?P<pk>\d+)/cabinets/add/$',
        view=DocumentAddToCabinetView.as_view(), name='document_cabinet_add'
    ),
    url(
        regex=r'^documents/multiple/cabinets/add/$',
        view=DocumentAddToCabinetView.as_view(),
        name='document_multiple_cabinet_add'
    ),
    url(
        regex=r'^documents/(?P<pk>\d+)/cabinets/remove/$',
        view=DocumentRemoveFromCabinetView.as_view(),
        name='document_cabinet_remove'
    ),
    url(
        regex=r'^documents/multiple/cabinets/remove/$',
        view=DocumentRemoveFromCabinetView.as_view(),
        name='multiple_document_cabinet_remove'
    ),
    url(
        regex=r'^documents/(?P<pk>\d+)/cabinets/$',
        view=DocumentCabinetListView.as_view(), name='document_cabinet_list'
    ),
]

urlpatterns = []
urlpatterns.extend(urlpatterns_cabinets)
urlpatterns.extend(urlpatterns_documents_cabinets)

api_urls = [
    url(
        regex=r'^cabinets/(?P<pk>[0-9]+)/documents/(?P<document_pk>[0-9]+)/$',
        view=APICabinetDocumentView.as_view(), name='cabinet-document'
    ),
    url(
        regex=r'^cabinets/(?P<pk>[0-9]+)/documents/$',
        view=APICabinetDocumentListView.as_view(), name='cabinet-document-list'
    ),
    url(
        regex=r'^cabinets/(?P<pk>[0-9]+)/$', view=APICabinetView.as_view(),
        name='cabinet-detail'
    ),
    url(
        regex=r'^cabinets/$', view=APICabinetListView.as_view(),
        name='cabinet-list'
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/cabinets/$',
        view=APIDocumentCabinetListView.as_view(), name='document-cabinet-list'
    ),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2019-12-10 04:36
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('web_links', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='weblink',
            name='label',
            field=models.CharField(db_index=True, help_text='A short text describing the web link.', max_length=96, verbose_name='Label'),
        ),
        migrations.AlterField(
            model_name='weblink',
            name='template',
            field=models.TextField(help_text='Template that will be used to craft the final URL of the web link. The {{ document }} variable is available to the template.', verbose_name='Template'),
        ),
    ]



from __future__ import unicode_literals

from django import forms
from django.utils.encoding import force_text
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _, ugettext

from mayan.apps.common.widgets import TextAreaDiv

from .models import DocumentPageContent


class DocumentContentForm(forms.Form):
    """
    Form that concatenates all of a document pages' text content into a
    single textarea widget
    """
    def __init__(self, *args, **kwargs):
        self.document = kwargs.pop('instance', None)
        super(DocumentContentForm, self).__init__(*args, **kwargs)
        content = []
        self.fields['contents'].initial = ''
        try:
            document_pages = self.document.pages.all()
        except AttributeError:
            document_pages = []

        for page in document_pages:
            try:
                page_content = page.content.content
            except DocumentPageContent.DoesNotExist:
                pass
            else:
                content.append(conditional_escape(force_text(page_content)))
                content.append(
                    '\n\n\n<hr/><div class="document-page-content-divider">- %s -</div><hr/>\n\n\n' % (
                        ugettext(
                            'Page %(page_number)d'
                        ) % {'page_number': page.page_number}
                    )
                )

        self.fields['contents'].initial = mark_safe(''.join(content))

    contents = forms.CharField(
        label=_('Contents'),
        widget=TextAreaDiv(
            attrs={
                'class': 'text_area_div full-height',
                'data-height-difference': 360
            }
        )
    )


class DocumentPageContentForm(forms.Form):
    contents = forms.CharField(
        label=_('Contents'),
        widget=TextAreaDiv(
            attrs={
                'class': 'text_area_div full-height',
                'data-height-difference': 360
            }
        )
    )

    def __init__(self, *args, **kwargs):
        document_page = kwargs.pop('instance', None)
        super(DocumentPageContentForm, self).__init__(*args, **kwargs)
        content = ''
        self.fields['contents'].initial = ''

        try:
            page_content = document_page.content.content
        except DocumentPageContent.DoesNotExist:
            pass
        else:
            content = conditional_escape(force_text(page_content))

        self.fields['contents'].initial = mark_safe(content)
